package alignment;

import java.io.*;
import java.util.Vector;

import ontology.HomologyOntology;
import ontology.OboOntology;
import ontology.OntologyInstance;
import visualization.Main;

// /!\ TODO LIST:

// - Mon algo de propagation actuel ne peut construire des homologies potentielles
//   qu'entre noeuds qui sont a egale distance du noeud de depart.
//   Autrement dit, si j'ai A<-B<-C dans une espece et A<-C dans une autre, si les deux
//   C sont homonymes, pas les A, mais que les A sont homologues, pour le moment je ne
//   peux pas construire cette homologie.
//   C'est cette possibilite de construction qui rend la propagation tres couteuse en
//   temps de calcul : je dois m'affranchir de la hierarchie dans une des deux ontologies.
//   Pour ca il faut que je recupere la liste de noeuds incidents jusqu'a une profondeur
//   donnee (pour l'ontologie de reference), et que je compare les incidents avec les
//   incidents de l'ontologie de travail (ou la je peux me promener par niveau).
//   Probablement qu'il faut que je favorise le score dans le cas d'incidents situes a la
//   meme distance des noeuds d'origine.

/**
 * A class that stores, for an organ, all the informations regarding the on-going ontology
 * alignment.
 *
 * @author Gilles Parmentier
 */
public class AnnotatedOrgan implements Serializable
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;
 
/**
  * Reference to the alignment algorithm. I wonder if this is wrong: I need this here
  * because a lot of the algorithmic part is located in this class, it's wrong imo.
  */
 HomologyMaker _maker;
 
 /**
  * The organ for which this object stores anootations.
  */
 OntologyInstance _oiOrgan;
 
 /**
  * The homology ontology.
  */
 HomologyOntology _oHomology;
 
 /**
  * The homology group the organ is in.
  */
 OntologyInstance _oiHomologyGroup;
 Vector <PairingScore> _putativeScores;
 Vector <OntologyInstance> _removedCandidates;
 Vector <Double> _removedCandidatesScores;
 Vector <Integer> _removedCandidatesStatus;

 final static int INVALIDATED_CHOICE = 0;
 final static int HOMOLOGY = 1;
 final static int DELAY_IGNORE = 2;
 final static int INVALIDATED_WITHOUT_PROPOSITION = 3;

 Vector <PairingScore> _sHomolog;

 AnnotatedOrgan (Main m, HomologyMaker hm, HomologyOntology ho, OntologyInstance organ)
 {
  _oiOrgan = organ;
  _oiHomologyGroup = null;
  _oHomology = ho;
  _putativeScores = new Vector<PairingScore> ();
  _removedCandidates = new Vector<OntologyInstance> ();
  _removedCandidatesScores = new Vector<Double> ();
  _removedCandidatesStatus = new Vector<Integer> ();
  _maker = hm;
  _sHomolog = new Vector<PairingScore> ();
  }

 /**
  * Validates the given <code>PairingScore</code> has being an homolog.
  * There are different things to consider: first, do we already have an homology group
  * for this organ. Second, does the reference organ belong to an existing homology group.
  * If both conditions are true then we have to merge the two existing groups.
  * @param ps The <code>PairingScore</code> to validate.
  */
 public void validateProposition (PairingScore ps, boolean strict)
 {
  System.out.print ("Validating " + _oiOrgan.getId ());

  // First we check if our reference organ belongs to an homology group:
  OntologyInstance reference_organ = ps.getOrganCandidate ();
  // We will have to check if ref_hog is null or not:
  OntologyInstance ref_hog = _oHomology.getHomologyGroupOfOrgan (reference_organ);

  System.out.println ("; with " + reference_organ.getId ());
  if (ref_hog == null) System.out.println ("-> that does not have an homology group");
  else System.out.println (" that has an homology group");

  if (_oiHomologyGroup == null)
     {
      System.out.println (_oiOrgan.getId () + " does not have an homology group yet.");
      // The current organ does not belong to an homology group.
      if (ref_hog != null)
         {
          // On a un groupe d'homologie pour l'organe de reference, on doit juste y
          // ajouter l'organe en cours.
          ref_hog.addProperty ("inv_" + ((HomologyOntology) _oHomology).getPartOfPropertyId (), _oiOrgan.getId ());

          // /!\ TODO : A FINIR
          // Il manque l'ajout de noms a la propriete name.
          }
      else
         {
          // Creation d'un nouveau groupe d'homologie :
    	  System.out.println ("Creating a new homology group...");
          String source_id = _oHomology.getNextHomologyGroupId ();

          OntologyInstance hg = new OntologyInstance (_oHomology, _oiOrgan.getName (),
                                        source_id, _oHomology.getHomologyGroupClassId (), new Vector <String> (), null);
          _oHomology.addInstance (hg);
          hg.addProperty ("inv_" + ((HomologyOntology) _oHomology).getPartOfPropertyId (), _oiOrgan.getId ());
          hg.addProperty ("inv_" + ((HomologyOntology) _oHomology).getPartOfPropertyId (), ps.getOrganCandidate ().getId ());

          // /!\ TODO : A FINIR
          // Il manque l'ajout de noms a la propriete name.
          }
      }
  else
     {
      System.out.println (_oiOrgan.getId () + " has an homology group.");
      // The current organ belongs to an homology group.
      if (ref_hog != null)
         {
          // C'est la le plus complique : il faut fusionner les deux groupes d'homologie.
          _oHomology.merge (ref_hog, _oiHomologyGroup);
          }
      else
         {
          // On ajoute l'organe de reference au groupe d'homologie de l'organe de travail.
          _oiHomologyGroup.addProperty ("inv_" + ((HomologyOntology) _oHomology)
                               .getPartOfPropertyId (), ps.getOrganCandidate ().getId ());

           // /!\ TODO : A FINIR
           // Il manque l'ajout de noms a la propriete name.
          }
      }
  _removedCandidatesScores.add (ps.getScore ());
  _removedCandidatesStatus.add (HOMOLOGY);
  ps.validate ();
  _sHomolog.add (ps);
  _putativeScores.remove (ps);
  _removedCandidates.add (ps.getOrganCandidate ());

  if (strict)
     {
      // Si l'homologie est stricte, aucun frere de l'organe de reference ne peut etre
      // propose comme homologue de l'organe de travail, et reciproquement.
      String p = ((OboOntology) _oiOrgan.getOntology()).getPartOfPropertyId (); // Pas de inv ici
      Vector <String> brothers = _oiOrgan.getBrothersInProperty (p);
      if (brothers.size () > 0)
         for (int i = 0; i < brothers.size (); i++)
             {
              AnnotatedOrgan ao = _maker.getAnnotatedOrganByOrganId ((String) brothers.get (i));
              ao.invalidatePossibility (reference_organ.getId ());
              }

      Vector <String> ref_brothers = reference_organ.getBrothersInProperty(p);
      if (ref_brothers.size () > 0)
         for (int i = 0; i < ref_brothers.size (); i++)
             this.invalidatePossibility ((String) ref_brothers.get (i));
      }

  this.dumpToConsole ();
  }

 /**
  * Handles the invalidation of a possible link between two instances.
  */
 protected void invalidatePossibility (String org_id)
 {
  // Verifier si org_id est dans la liste des propositions, si c le cas il faut le
  // supprimer
  for (int i = 0; i < _putativeScores.size (); i++)
      {
       PairingScore ps = _putativeScores.get (i);
       if (ps.getOrganCandidate ().getId ().equals (org_id))
          {
           _putativeScores.remove (ps);
           }
       }
  _removedCandidatesScores.add (- 1.0);
  _removedCandidatesStatus.add (INVALIDATED_CHOICE);
  _removedCandidates.add (_maker.getReferenceOntologyInstanceById (org_id));
  //this.dumpToConsole ();
  }

 /**
  * Removes the current best appariement choice from the candidate list. This organ will
  * no longer be a putative homolog of the current organ.
  * @param ps The candidate that has been invalidated.
  * @return A <code>PairingScore</code>, the new best candidate.
  */
 public PairingScore invalidateProposition (PairingScore ps)
 {
  System.out.println ("Invalidating " + _oiOrgan.getId ());
  System.out.println ("With " + ps.getOrganCandidate ().getId ());

  _removedCandidatesScores.add (ps.getScore ());
  _removedCandidatesStatus.add (INVALIDATED_CHOICE);

  // Add the former best choice to the ignore list:
  _removedCandidates.add (ps.getOrganCandidate ());
  // Remove it from the list of choices:
  ps.invalidateSelf ();
  _putativeScores.remove (ps);

  PairingScore newBest = null;
  double best_score = -1;
  for (int i = 0; i < _putativeScores.size (); i++)
      {
       PairingScore s = (PairingScore) _putativeScores.elementAt (i);
       if (s.getScore () > best_score)
          {
           best_score = s.getScore ();
           newBest = s;
           }
       }

  if (newBest == null)
     {
      System.err.println ("invalidateProposition; " + _oiOrgan.getId () + " newBest is null");
      System.err.println ("\t_putativeScores size was: " + _putativeScores.size ());
      System.err.println ("\torgan to invalidate was: " + ps.getOrganCandidate().getId ());
      }

  //this.dumpToConsole ();
  return newBest;
  }

 /**
  * 
  * @param ps
  * @return
  */
 public PairingScore delayProposition (PairingScore ps)
 {
  System.out.println ("Delaying " + _oiOrgan.getId ());
  System.out.println ("With " + ps.getOrganCandidate ().getId ());
  int best_pos = -1;
  _removedCandidatesScores.add (ps.getScore ());
  _removedCandidatesStatus.add (DELAY_IGNORE);
  // J'ai besoin de la positon du PairingScore en cours pour pouvoir l'eliminer de la
  // liste ensuite :
  for (int i = 0; i < _putativeScores.size (); i++)
      {
       PairingScore s = (PairingScore) _putativeScores.elementAt (i);
       if (ps.getOrganCandidate ().getId ().equals (s.getOrganCandidate ().getId ()))
          {
  best_pos = i;
   break;
       }
    }
  // Add the former best choice to the ignore list:
  _removedCandidates.add (ps.getOrganCandidate ());
  // Remove it from the list of choices:
  _putativeScores.remove (best_pos);

  PairingScore newBest = null;
  double best_score = -1;
  for (int i = 0; i < _putativeScores.size (); i++)
      {
       PairingScore s = (PairingScore) _putativeScores.elementAt (i);
       if (s.getScore () > best_score)
          {
           best_score = s.getScore ();
           newBest = s;
           }
       }

  if (newBest == null)
     {
      System.err.println ("delayProposition; " + _oiOrgan.getId () + " newBest is null");
      System.err.println ("\t_putativeScores size was: " + _putativeScores.size ());
      System.err.println ("\torgan to invalidate was: " + ps.getOrganCandidate().getId ());
      }

  //this.dumpToConsole ();
  return newBest;
  }

 /**
  * @param oiHomonym The new organ to add to the homonyms list.
  */
 void addHomonym (OntologyInstance oiHomonym)
 {
  // Check if oiHomonym is in the discards list:
  boolean discard = false;
  for (int i = 0; i < _removedCandidates.size (); i++)
      if (_removedCandidates.get(i).getId ().equals (oiHomonym.getId ()))
         {
          discard = true;
          break;
          }

  if (! discard)
     {
      // Creer un nouveau score :
	  PairingScore s = new PairingScore (_oiOrgan);
      s.setInitialCandidate (oiHomonym);
      _putativeScores.add (s);
      }
  }

 /**
  * Create an initial candidate by homonymy, but using at least one of the synonyms. The
  * used synonym(s) is (are) the non null one(s)
  * @param oiCandidate
  * @param s1
  * @param s2
  */
 void addSynonym (OntologyInstance oiCandidate, String s1, String s2)
 {
 // Check if oiCandidate is in the discards list:
 boolean discard = false;
 for (int i = 0; i < _removedCandidates.size (); i++)
     if (_removedCandidates.get(i).getId ().equals (oiCandidate.getId ()))
        {
         discard = true;
         break;
         }

 for (int i = 0; i < _putativeScores.size (); i++)
     if (oiCandidate.getId ().equals (_putativeScores.get (i).getOrganCandidate ().getId ()))
        {
   discard = true;
   break;
         }

 if (! discard)
 {
    if ((s1 != null) && (s2 != null))
       {
	//System.out.println ("Setting synonym-synonym");
        PairingScore s = new PairingScore (_oiOrgan);
        s.setInitialSynonym12Candidate (oiCandidate, s1, s2);
        _putativeScores.add (s);
        }
    else
       if (s1 != null)
          {
           //System.out.println ("Setting synonym-name");
           PairingScore s = new PairingScore (_oiOrgan);
           s.setInitialSynonym1Candidate (oiCandidate, s1);
           _putativeScores.add (s);
           }
       else
          if (s2 != null)
             {
              //System.out.println ("Setting name-synonym");
              PairingScore s = new PairingScore (_oiOrgan);
              s.setInitialSynonym2Candidate (oiCandidate, s2);
              _putativeScores.add (s);
              }
    }
    else
       {
        //System.out.println ("Discarded!");
        }
  }

 /**
  * Sets an homolog for the curent organ.
  * @param hg The homology group the homolog is belonging to.
  * @param homolog The homologous organ.
  */
 void setHomolog (OntologyInstance hg, OntologyInstance homolog)
 {
  if (_oiHomologyGroup == null)
     {
      _oiHomologyGroup = hg;
      // Il faut que je cree un PairingScore pour le placer dans la liste des ignore.
      PairingScore ps = new PairingScore (this._oiOrgan);
      ps.setCandidate (homolog);
      _removedCandidatesScores.add (ps.getScore ());
      _removedCandidatesStatus.add (HOMOLOGY);
      _sHomolog.add (ps);
      ps.setCandidate (homolog);
      ps.validate ();
      _removedCandidates.add (homolog);
      }
  else
     {
      // Cette fonction peut etre appelee plusieurs fois. A priori je pense que hg sera le
      // meme a chaque fois, mais il vaut mieux verifier. En theorie ce cas n'est pas
      // atteint.
      System.err.println ("setHomolog (): reaching  impossible case for " + _oiOrgan.getId());
      }
  }

 /**
  * 
  */
public PairingScore getHomologScore () { return _sHomolog.get (0); }

 /**
  * @param oiCandidate The member of the reference ontology that is added as a candidate
  * for coupling to the current instance.
  */
 public PairingScore addCandidate (OntologyInstance oiCandidate)
 {
  if (isValidChoice (oiCandidate))
  {
  PairingScore s = new PairingScore (_oiOrgan);
  s.setCandidate (oiCandidate);
  _putativeScores.add (s);
  return s;
  }
  return null;
  }

 /**
  * 
  */
 public void addPairingScore (PairingScore ps) { _putativeScores.add (ps); }
 
 /**
  * Debugging function, prints the some infomation for tis <code>AnnotatedOrgan</code> to
  * the console.
  * @return Returns true if the <code>AnnotatedOrgan</code> has at least one candidate.
  */
 public boolean dumpToConsole ()
 {
  System.out.println ("***AnnotatedOrgan***");
  System.out.println ("\tid: " + _oiOrgan.getId ());
  System.out.println ("\tname: " + _oiOrgan.getName ());
  boolean ret_val;
  if (_putativeScores.size () > 0)
     {
      double best_score = ((PairingScore) _putativeScores.elementAt (0)).getScore ();
      int best_pos = 0;
      for (int i = 1; i < _putativeScores.size (); i++)
          if (((PairingScore) _putativeScores.elementAt (i)).getScore () > best_score) 
             {
              best_pos = i;
              best_score = ((PairingScore) _putativeScores.elementAt (i)).getScore ();
              }
      PairingScore s = (PairingScore) _putativeScores.elementAt (best_pos);
      System.out.println ("\tscore: " + s.getScore ());
      System.out.println ("\tBest candidate:");
      OntologyInstance oic = s.getOrganCandidate ();
      System.out.println ("\t  id: " + oic.getId ());
      System.out.println ("\t  name: " + oic.getName ());
      ret_val = true;
      // Pour debug bourrin j'ajoute la liste de tous les appariements :
      System.out.println ("\tComplete list of candidates:");
      for (int i = 0; i < _putativeScores.size (); i++)
          {
           PairingScore ps = (PairingScore) _putativeScores.get (i);
           OntologyInstance oi = ps.getOrganCandidate ();
           System.out.println ("\t  id: " + oi.getId () + ", name: " + oi.getName () + ", score: " +ps.getScore ());
           }
      }
  else
     {
      System.out.println ("\t=> no candidate for homology");
      ret_val = false;
      }
  
  if (_removedCandidates.size () > 0)
     {
      System.out.println ("\tRemoved candidates:");
      for (int i = 0; i < _removedCandidates.size (); i++)
          {
           System.out.println ("\t  id:" + _removedCandidates.get(i).getId() + " " +
        		                                    _removedCandidates.get(i).getName ());
           }
      }
  else
     {
      System.out.println ("No removed candidates");
      }
  return ret_val;
  } // public boolean dumpToConsole ()

 // Getters:

 /**
  * @return the name of the organ.
  */
 String getName () { return _oiOrgan.getName (); }

 HomologyMaker getHomologyMaker () { return _maker; }

 HomologyOntology getHomologyOntology () { return _oHomology; }
 
 public OntologyInstance getOrgan () { return _oiOrgan; }

 String getOrganId () { return _oiOrgan.getId (); }

 public Vector <PairingScore> getScores ()
 { if (isValidated ()) return _sHomolog; else return _putativeScores; }


 public void setScores (Vector <PairingScore> vps) { _putativeScores = vps; }

 /**
  * @return Returns the best score of the putative homologs candidates.
  */
 public double getBestScore ()
 {
  if (_putativeScores.size () == 0) return 0;
  double best_score = ((PairingScore) _putativeScores.elementAt (0)).getScore ();
  for (int i = 1; i < _putativeScores.size (); i++)
      {
       PairingScore s = (PairingScore) _putativeScores.elementAt (i);
       if (s.getScore () > best_score)
          best_score = s.getScore ();
       }
  return best_score;
  }

 /**
  * Returns the best <code>PairingScore</code> for this AnnotatedOrgan.
  * Note: It should probably be adapted to the case where this <code>AnnotatedOrgan</code>
  * has an homolog validated by some user.
  * @return The <code>PairingScore</code> with the highest score.
  */
public PairingScore getBestScoreObject ()
{
 if (_putativeScores.size () == 0) { System.err.println ("Warning: putative scores is empty and we are requesting the best one!"); return null; }
 PairingScore s = (PairingScore) _putativeScores.elementAt (0);
 double best_score = s.getScore ();
 int best_pos = 0;
 for (int i = 1; i < _putativeScores.size (); i++)
     if (((PairingScore) _putativeScores.elementAt (i)).getScore () > best_score) 
        {
         best_pos = i;
         best_score = ((PairingScore) _putativeScores.elementAt (i)).getScore ();
         }
 s = (PairingScore) _putativeScores.elementAt (best_pos);

 return s;
 }

 public int getNbAppariements () { return _putativeScores.size (); }

 public int getNbHomologs () { return _sHomolog.size (); }
 
 public void writeDiscards (BufferedWriter bw) throws IOException
 {
  String sid = _oiOrgan.getId ();
  for (int i = 0; i < _removedCandidates.size (); i++)
      {
       OntologyInstance oiremoved = _removedCandidates.get(i);
       bw.write (sid + " " + oiremoved.getId ());
       bw.newLine ();
       }
  }

 // Etend la fonction du dessus pour completer l'information et permettre la realisation
 // de statistiques sur le comportement de la fonction de score.
 public void writeValues (BufferedWriter bw, Vector<String []> v) throws IOException
 {
  String sid = _oiOrgan.getId ();
  if (v != null)
     {
      // Avant d'ecrire un couple je dois verifier s'il est deja present dans la liste des valeurs ecrites.
      for (int i = 0; i < _removedCandidates.size (); i++)
          {
           OntologyInstance oiremoved = _removedCandidates.get (i);
           String rsid = oiremoved.getId ();
           boolean existing = false;
           for (int j = 0; j < v.size (); j++)
	           if ((sid.equals (v.get (j) [0])) && (rsid.equals (v.get (j) [1])))
	              { existing = true; j = v.size (); }
           if (! existing)
              {
               bw.write (sid +  "\t" + oiremoved.getId () + "\t" +
                                           _removedCandidatesStatus.get (i)+ "\t" +
                                   _removedCandidatesScores.get (i).doubleValue());
               bw.newLine ();
               }
           }
      return ;
      }
  else
     {
      for (int i = 0; i < _removedCandidates.size (); i++)
          {
           OntologyInstance oiremoved = _removedCandidates.get(i);
           bw.write (sid +  "\t" + oiremoved.getId () + "\t" +
                                                  _removedCandidatesStatus.get (i)+ "\t" +
                                          _removedCandidatesScores.get (i).doubleValue());
           bw.newLine ();
           }
      }
  }
 
 // Setters:

 void setHomologyGroup (OntologyInstance hg) { _oiHomologyGroup = hg; }

 // Testers:
 
 public boolean isValidated ()
 {
  if (_putativeScores.size () == 0) return false;
  return getBestScoreObject ().isValidated ();
  }

 public boolean isPutative () { return _sHomolog == null; }

 public boolean hasCandidate () { return (_putativeScores.size () > 0); }
 
 public PairingScore hasCandidate (OntologyInstance oi)
 {
  for (int i = 0; i < this._putativeScores.size (); i++)
      {
       //System.out.println ("Checking if " + oi.getId () + " is equal to " + _putativeScores.get (i).getOrganCandidate ().getId ());
       if (_putativeScores.get (i).getOrganCandidate ().getId ().equals (oi.getId ()))
          return _putativeScores.get (i);
       }
  return null;
  }

 public boolean isValidChoice (OntologyInstance oi)
 {
  for (int i = 0; i < _removedCandidates.size (); i++)
      if (_removedCandidates.get (i).getId ().equals (oi.getId ()))
         return false;
  return true;
  }

 public void addDiscardedChoice(OntologyInstance instance)
 {
  _removedCandidates.add (instance);
  _removedCandidatesScores.add (- 1.0);
  _removedCandidatesStatus.add (INVALIDATED_WITHOUT_PROPOSITION);
  }
 }; // class AnnotatedOrgan