package alignment;


import java.io.*;
import java.util.Vector;

import ontology.OntologyInstance;
import ontology.SpeciesOntology;


/**
 * Simple scoring scheme for the pairing of two organs. We take into account the
 * similarity in the number of son and parents for both instancces.
 *
 * @author Gilles Parmentier
 */

public class OrganHierarchyPairingScorer extends OrganSimplePairingScorer implements Serializable
{
 private static final long serialVersionUID = 1L;

 /**
  * Scoring function.
  * @return  The score of the pairing, 5.0 if the organs names are equaled, 0.5 otherwise.
  */
 public double score (OntologyInstance oi1, OntologyInstance oi2)
 {
  // Ici je veux decomposer le score en deux parties :
  // - Une tenant compte du pattern matching entre les noms des deux instances.
  // - Une tenant compte de la hierarchie environnante dans les ontologies pour les deux instances. Cette partie pourrait etre tres complexe. Pour l'instant je me contente de comparer le nombre de fils et le nombre de parents.

  // Name part:
  double pattern_part = 0.0;
  if (oi1.getName ().equals (oi2.getName ()))
     pattern_part = 5.0;
  else
     pattern_part = 0.0;

  // Hierarchy part:
  Vector <String> o1_parent = oi1.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) oi1.getOntology ()).getPartOfPropertyId ());
  int oi1_nb_parent = 0;
  if (o1_parent != null) oi1_nb_parent = o1_parent.size ();
  Vector <String> o2_parent = oi2.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) oi2.getOntology ()).getPartOfPropertyId ());
  int oi2_nb_parent = 0;
  if (o2_parent != null) oi2_nb_parent = o2_parent.size ();
  double parent_ratio = 0;
  if (oi1_nb_parent >= oi2_nb_parent)
     parent_ratio = (oi2_nb_parent + 1) / (oi1_nb_parent + 1);
  else parent_ratio = (oi1_nb_parent + 1) / (oi2_nb_parent + 1);

  Vector <String> o1_child = oi1.getObjectOfPropertyByPropertyIdAsVector ("inv_" + ((SpeciesOntology) oi1.getOntology ()).getPartOfPropertyId ());
  int oi1_nb_child = 0;
  if (o1_child != null) oi1_nb_child = o1_child.size ();
  Vector <String> o2_child = oi2.getObjectOfPropertyByPropertyIdAsVector ("inv" + ((SpeciesOntology) oi2.getOntology ()).getPartOfPropertyId ());
  int oi2_nb_child = 0;
  if (o2_child != null) oi2_nb_child = o2_child.size ();
  double child_ratio = 0;
  if (oi1_nb_child >= oi2_nb_child)
     child_ratio = (oi2_nb_child + 1) / (oi1_nb_child + 1);
  else child_ratio = (oi1_nb_child + 1) / (oi2_nb_child + 1);

  return pattern_part + parent_ratio + child_ratio;
  }
 }