package alignment;

import java.util.Vector;

import ontology.OntologyInstance;
import ontology.SpeciesOntology;

/**
 * Given a <code>PairingScore</code>, this class looks for all the similar
 * <code>AnnotatedScore</code> in the existing propositions.
 * 
 * @author Gilles Parmentier
 */
// TODO:
// Question: maybe we have to look for putative pairs that do not exist yet?
public class SimilarPairFinder
{
 Vector <PairingScore> _results;

 public SimilarPairFinder (PairingScore ps, Vector <AnnotatedOrgan> v)
 {
  OntologyInstance oi_left = ps.getInstance ();
  OntologyInstance oi_right = ps.getOrganCandidate ();
  _results = findSimilars (oi_left, oi_right, v);

  Vector <PairingScore> tmp_res = new Vector <PairingScore> ();
  for (int i = 0; i < _results.size (); i++)
   if (hasEquivalentParents (oi_left.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) oi_left.getOntology ()).getPartOfPropertyId ()), oi_right.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) oi_right.getOntology ()).getPartOfPropertyId ()), _results.get (i)))
      tmp_res.add (_results.get (i));
  _results = tmp_res;
  }

 /**
  * Find all the pairs that have members with the same name than <code>oi_left</code> and
  * <code>oi_right</code>
  * @param oi_left
  * @param oi_right
  * @param v
  * @return
  */
 // Je cherche tous les termes de l'ontologie de gauche qui porte le meme nom que celui de
 // la pair validee. Pour chacun de ces termes je verifie s'il a une proposition qui porte
 // le meme nom que mon terme de droite.
 // Si c'est le cas, je remplie ma premiere condition de similitude.
 protected Vector <PairingScore> findSimilars (OntologyInstance oi_left, OntologyInstance  oi_right, Vector <AnnotatedOrgan> v)
 {
  Vector <PairingScore> res = new Vector <PairingScore> ();
  for (int i = 0; i < v.size (); i++)
  {
   AnnotatedOrgan ao = v.get (i);
   OntologyInstance ao_left = ao.getOrgan ();
   if (oi_left.getName ().equals (ao_left.getName ()) && (oi_left.getId () != ao_left.getId ()))
      {
       System.err.println ("Found same name: " + ao_left.getId ());
       // Verifier si dans la liste de proposition certaines portent le meme nom que oi_right.
       Vector <PairingScore> vps = ao.getScores ();
       for (int j = 0; j < vps.size (); j++)
           {
            System.err.println ("Comparing: " + oi_right.getName () + " " + vps.get (j).getOrganCandidate ().getName ());
            if (oi_right.getName ().equals (vps.get (j).getOrganCandidate ().getName ()))
               {
                res.add (vps.get (j));
                System.err.println ("Adding: " + vps.get (j).getInstance ().getId () + "&" + vps.get (j).getOrganCandidate ().getId ());
                }
            else
               {
                System.err.println ("Not adding: " + vps.get (j).getInstance ().getId () + "&" + vps.get (j).getOrganCandidate ().getId ());
                }
            }
       }
   }
  return res;
  }

 protected boolean hasEquivalentParents (Vector <String> left_parents, Vector <String> right_parents, PairingScore ps)
 {
  OntologyInstance left = ps.getInstance ();
  OntologyInstance right = ps.getOrganCandidate ();

  Vector <String> lp = left.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) left.getOntology ()).getPartOfPropertyId ());
  // In the general case, both left_parents and lp may have several members. Currently I
  // juste chec fort at least one common member.
  boolean left_ok = false;
  //System.err.println (lp.size () + " " + left_parents.size ());
  for (int i = 0; i < lp.size (); i++)
   for (int j = 0; j < left_parents.size (); j++)
   {
    //System.err.println (left.getOntology ().getInstanceById (lp.get (i)).getName () + "" + left.getOntology ().getInstanceById (left_parents.get (j)).getName ());
    if (left.getOntology ().getInstanceById (lp.get (i)).getName ().equals (left.getOntology ().getInstanceById (left_parents.get (j)).getName ()))
    {
     //System.err.println ("true!");
     left_ok = true;
     break;
    }
   }
  Vector <String> lr = right.getObjectOfPropertyByPropertyIdAsVector (((SpeciesOntology) right.getOntology ()).getPartOfPropertyId ());
  boolean right_ok = false;
  for (int i = 0; i < lr.size (); i++)
   for (int j = 0; j < right_parents.size (); j++)
    if (right.getOntology ().getInstanceById (lr.get (i)).getName ().equals (right.getOntology ().getInstanceById (right_parents.get (j)).getName ()))
    {
     right_ok = true;
     break;
    }
  //System.err.println ("Testing: " + left.getName () + " " + right.getName () + left_ok + " " + right_ok);
  return (left_ok && right_ok);
  }

 public Vector <PairingScore> getResults () { return _results; }
 }
