package alignment;

import java.io.Serializable;
import java.util.Enumeration;
import java.util.Hashtable;
import java.util.Vector;


import ontology.OntologyInstance;

// TODO: Verifier ce que je fais lorsque je diminue le score de deux noms qui ont t
// invalids. Il semblerait que je mette un facteur x0.9 que les termes soient  homonymes
// ou pas. Ce n'est pas faux, mais ce n'est pas ce que je prevoyais a la base. A priori ca
// n'a aucune influence car le facteur ne devrait etre vrifi que lorsque l'on est en
// presence de termes homonymes.

public class SyntaxicPairingScorer extends OrganSimplePairingScorer implements Serializable
{
 private static final long serialVersionUID = 1L;
 Hashtable <String, Double> _working, _reference;
 //double _working_mean, _reference_mean;

 public SyntaxicPairingScorer ()
 {
  //System.out.println ("Creating a syntaxicPairingScorer");
  _working = new Hashtable<String, Double> ();
  _reference = new Hashtable<String, Double> ();
  }

 /**
  * 
  */
 public void init (Vector <OntologyInstance> working_instances, Vector <OntologyInstance> reference_instances)
 {
  fill (_working, working_instances);
  fill (_reference, reference_instances);
  
  //  (base * nb_mots_communs * max_des_poids) / max (longueurs)
  // max_des_poids : produits des poids dans les deux especes, que pour les mots communs

  // Dump des resultats pour verif :
  //System.out.println ("Working:");
  //_working_mean = 0;
  for (Enumeration <String> e = _working.keys () ; e.hasMoreElements() ;)
      {
       String elem = e.nextElement ();
       //_working.put (elem, Math.max (1 / (1 + Math.log10 (_working.get (elem))), 0.5));
       _working.put (elem, 1 / (1 + Math.log10 (_working.get (elem))));
       //System.out.println (elem + "\t" + _working.get (elem));
       //_working_mean += _working.get (elem);
       }
  //_working_mean = _working_mean / _working.size ();
  
  //System.out.println ("Reference:");
  for (Enumeration <String> e2 = _reference.keys () ; e2.hasMoreElements() ;)
      {
	   String elem = e2.nextElement ();
       //_reference.put (elem, Math.max (1 / (1 + Math.log10 (_reference.get (elem))), 0.5));
       _reference.put (elem, 1 / (1 + Math.log10 (_reference.get (elem))));
       //System.out.println (elem + "\t" + _reference.get (elem));
       //_reference_mean += _reference.get (elem);
       }
  //_reference_mean = _reference_mean / _working.size ();
  
  //System.out.println ("working mean frequence: " + _working_mean);
  //System.out.println ("reference mean frequence: " + _reference_mean);
  }

 public void fill (Hashtable <String, Double> h, Vector <OntologyInstance> v)
 {
  for (int i = 0; i < v.size (); i++)
      {
       OntologyInstance oi = v.get (i);
       String [] words = oi.getName ().split (" ");
       for (int j = 0; j < words.length; j++)
           {
            // Si c'est une valeur numerique, je veux etre certain qu'elle est stockee dans une forme canonique.
            if (isNumeric (words [j]))
               {
                Double d = new Double (words [j]);
                words [j] = String.valueOf (d);
                }
            // Verifier si le mot a deja ete stocke. Si c'est le cas on incremente son compteur de 1. Sinon on le rajoute.
            if (h.containsKey (words [j]))
               {
                Double value = h.get (words [j]);
                value++;
                h.put (words [j], value);
                }
            else
                h.put (words [j], new Double (1));
            }
       }
  }

 public double score (String oi1, String oi2)
 {
  //System.err.println ("Scoring " + oi1 + " and " + oi2);
  double homonymy_score = 50.0;
  String [] words1 = oi1.split(" ");
  String [] words2 = oi2.split (" ");
  int nb_common = 0;
  double max_weight = 0;
 
  // This one is to handle the factor of the decreased pairs:
  double modificator_factor = 1;

  for (int i = 0; i < words1.length; i++)
      {
       if (isNumeric (words1 [i]))
          {
           Double d = new Double (words1 [i]);
           words1 [i] = String.valueOf (d);
           }
       for (int j = 0; j < words2.length; j++)
           {
            if (isNumeric (words2 [j]))
               {
                Double d = new Double (words2 [j]);
                words2 [j] = String.valueOf (d);
                }
            if (words1 [i].equals (words2 [j]))
               {
                nb_common++;
                double current_max = (_working.get (words1 [i]) == null ? 1 / (1 + Math.log10 (1)) : _working.get (words1 [i])) * (_reference.get (words2 [j]) == null ? 1 / (1 + Math.log10 (1)) : _reference.get (words2 [j]));
                if (current_max > max_weight) max_weight = current_max;
                Double d = (_pairs_modificators.get (words1 [i])) == null ?
                           null : (_pairs_modificators.get (words1 [i])).get (words2 [j]);
                if (d != null) modificator_factor = modificator_factor * d.doubleValue ();
                }
            }
       }
  //System.out.println ("Score for " + oi1 + " and " + oi2 + " is: " +
  //     homonymy_score * nb_common * max_weight / Math.max (words1.length, words2.length));
  return homonymy_score * nb_common * modificator_factor * max_weight / (Math.max (words1.length, words2.length));
  }
 }