package datalayer;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.util.List;
import java.util.Vector;

import ontology.HomologyOntology;
import ontology.OntologyInstance;

public class OboHomologyWriter
{
 /**
  * The homology ontology that will be written on disk.
  */
 HomologyOntology _o;

 /**
  * The <code>BufferedWriter</code> used to write the homology ontology.
  */
 BufferedWriter _bw;

 public OboHomologyWriter (String filename, HomologyOntology o)
 {
  _o = o;
  try
     {
      FileWriter file = new FileWriter (new File (filename));
      _bw = new BufferedWriter (file);

      writeHeader ();

      writeHasMemberRelationship ();

      // Write all instances:
      List <OntologyInstance> instances = _o.getInstances ();
      for (int i = 0; i < instances.size (); i++)
          {
           OntologyInstance oi = instances.get (i);
           writeTerm (oi);
           }
      _bw.close ();
      }
  catch (Exception e) { e.printStackTrace(); }
  }

 /**
  * 
  * @throws Exception
  */

 private void writeHeader () throws Exception
 {
  _bw.write ("format-version: 1.2");
  _bw.newLine ();
  _bw.write ("date: ");
  _bw.newLine ();
  _bw.write ("auto-generated-by: HomolOnto 0.1");
  _bw.newLine ();
  _bw.write ("default-namespace: anatomical_homology_ontology");
  _bw.newLine ();
  _bw.newLine ();
  }

 /**
  * 
  */
 private void writeTerm (OntologyInstance oi) throws Exception
 {
  Vector <String> v = oi.getObjectOfPropertyByPropertyIdAsVector ("inv_" + _o.getPartOfPropertyId ());
  if (v != null)
  {
  _bw.write ("[Term]");
  _bw.newLine ();
  _bw.write ("id: " + oi.getId ());
  System.err.println ("Writing " + oi.getId ());
  _bw.newLine ();
  _bw.write ("name: " + oi.getName ());
  _bw.newLine ();

  // List all the members of this group:
  for (int i = 0; i < v.size (); i++)
      {
       _bw.write ("relationship: has_member " + v.get(i));
       _bw.newLine ();
       }

  _bw.newLine ();
  }
  }

 /**
  * 
  */
 private void writeHasMemberRelationship () throws Exception
 {
  _bw.write("[Typedef]");
  _bw.newLine ();
  _bw.write("id: OGEE:has_member");
  _bw.newLine ();
  _bw.write("name: has_member");
  _bw.newLine ();
  _bw.write("is_a: OBO_REL:relationship");
  _bw.newLine ();
  _bw.write("def: C has_member C', C is an homology group and C' is a biological object");
  _bw.newLine ();
  _bw.write("comment: We leave open the possibility that an homology group is a ");
  _bw.write ("biological object. Thus, an homology group C may have C' has_member, ");
  _bw.write ("with C' being an homology group.");
  _bw.newLine ();
  _bw.write("is_transitive: true");
  _bw.newLine ();
  _bw.write("is_anti_symmetric: true");
  _bw.newLine ();
  _bw.newLine ();
  }
 }
