package datalayer;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;
import java.util.Vector;
import java.util.Map.Entry;

public class OboReader
{
 String _filename;
 BufferedReader _handle;
 Hashtable <String, Vector <String>> _complete_descriptions;

 Hashtable <String, Hashtable <String, String>> _triplets;

 private final static int NO_TOKEN = 0;
 private final static int TERM_TOKEN = 1;
 private final static int TYPEDEF_TOKEN = 2;

 public OboReader (String f)
 {
  _filename = f;
  _complete_descriptions = new Hashtable <String, Vector <String>> ();
  try {
       _handle = new BufferedReader (new FileReader (new File (_filename)));
       _triplets = new Hashtable <String, Hashtable <String, String>> ();
       read_header ();
       int token_type;
       while ((token_type = getNextTokenType ()) != NO_TOKEN)
             {
    	      if (token_type  == TERM_TOKEN)
    	         {
                  Vector <String> desc = readTerm ();
                  String id = desc .get (desc.size () - 1);
                  desc.remove (desc.size () - 1);
                  _complete_descriptions.put (id, desc);
    	          }
    	      if (token_type == TYPEDEF_TOKEN)
    		     readTypedef ();
              }
       //System.err.println ("Read: " + _triplets.size () + " tokens");
       infer_part_of ();
       }
  catch (FileNotFoundException e) { e.printStackTrace(); }
  }

 private void read_header ()
 {
  try {
       String str = _handle.readLine () ;
       while (! str.equals (""))
             {
              str = _handle.readLine ();
              //System.err.println (str);
              }
       }
  catch (IOException e) { e.printStackTrace(); }
  }

 private void readTypedef ()
 {
  String str;
  try {
  str = _handle.readLine ();
   while ((str != null) && ! str.equals ("")) str = _handle.readLine () ; }
  catch (IOException e) {}
  }

 private Vector <String> readTerm ()
 {
  String str;
  Hashtable <String, String> token = new Hashtable <String, String> ();
  int nb_syn = 0;
  Vector <String> all_lines = new Vector <String> ();
  String id = null;
  try {
       boolean is_obsolete = false;
       while (! (str = _handle.readLine ()).equals (""))
             {
              all_lines.add (str);
              //System.err.println (str);
              String [] words = str.split (": ");
              if (words [0].equals("id"))
                 {
                  token.put ("id", words [1]);
                  id = words [1];
                  System.out.println ("Read id: " + words [1]);
                  }
              else
                 if (words [0].equals ("name"))
                    {
                     token.put ("name", words [1]);
                     //System.err.println ("Read name: " + words [1]);
                     }
                 else
                    if (words [0].equals ("namespace"))
                       {
                        token.put ("namespace", words [1]);
                        }
                    else
                       if (words [0].equals ("xref_analog"))
                          {
                           token.put ("xref_analog", words [1]);
                           }
                       else
                          if (words [0].equals ("def"))
                             {
                              if (words.length == 2) token.put ("def", words [1]);
                        	  else {
                        	    //System.err.println ("Warning; TODO: multiple statement definition");
                        	    }
                              }
                          else
                             if (words [0].equals ("relationship"))
                                {
                                 if (words.length == 2)
                                    {
                                     String [] w2 = words [1].split(" ", 2);
                                     String [] w3 = w2 [1].split (" ! ");
                                     if ((! w3 [0].equals ("XUO:0000010")) && (! w3 [0].equals ("XUO:0000006")))
                                        {
                                         token.put(w2 [0], w3 [0]);
                                         //System.err.println ("relationship: " + w2 [0] + ":" + w3 [0]);
                                         }
                                     }
                                 else
                                    {
                                  //System.err.println ("Warnig; TODO: weird relationship line");
                                  }
                                 }
                             else
                                if (words [0].equals ("is_a"))
                                   {
                                    if (words.length == 2)
                                       {
                                        String [] w2 = words [1].split (" ! ");
                                        if (! w2 [0].equals ("XUO:0000004"))
                                        {
                                        token.put ("is_a", w2 [0]);
                                        //System.err.println ("is_a: " + w2 [0]);
                                        }
                                        }
                                    else
                                       {
                                     //System.err.println ("Warnig; TODO: weird is_a line");
                                     }
                                    }
                                else
                                   if (words [0].equals ("synonym"))
                                      {
                                       // Does not work, we may have more than one synonym.
                                       words = words [1].split("\"");
                                       token.put ("synonym:" + ++nb_syn, words [1]);
                                       //System.err.println ("Read synonym: " + words [1]);
                                       }
                                   else
                                      if (words [0].equals ("is_obsolete"))
                                         {
                                          is_obsolete = true;
                                          }
                                      else
                                         if (words [0].equals ("comment"))
                                            {
               
                                             }
                                      else
                                         {
                                          System.err.println ("Unrecognized string: " + str);
                                          }
              }

       if (! is_obsolete)
          _triplets.put ((String) token.get ("id"), token);
       } catch (IOException e) { e.printStackTrace(); }

  all_lines.add (id);
  return all_lines;
  }

 private int getNextTokenType ()
 {
  int type = NO_TOKEN;
  String str;
  try {
      str = _handle.readLine ();
      //System.err.println ("token type: " + str);
      if (str == null)
    	  return type;
      else
      if (str.equals ("[Term]"))
 return TERM_TOKEN;
      else
          if (str.equals ("[Typedef]"))
     return TYPEDEF_TOKEN;
       } catch (IOException e) { e.printStackTrace(); }
  return type;
  }

 public Hashtable <String, Hashtable <String, String>> getTriplets ()
 {
  return _triplets;
  }

 // Due to the new formalization associated with CARO we have to do some
 // modifications to what have been read. Namely, some organs are link to the
 // hierarchy only through develops_from relations. Our current choice is that if we
 // have
 // organ1 <-(part_of) organ x <-(develops_from) organ y
 // AND organ y does not have any part_of relationship then we infer:
 // organ1 <-(part_of) organ y

 // It should be noticed that if several organs are linked in the same way to A, we
 // will have troubles during propagation: some obviously wrong pairings will be
 // built.
 private void infer_part_of ()
 {
  //System.err.println ("infer_part_of ()");
  Set <Entry <String, Hashtable <String, String>>> s = _triplets.entrySet ();
  Iterator <Entry <String, Hashtable <String, String>>> it = s.iterator ();
  while (it.hasNext ())
        {
         Map.Entry <String, Hashtable <String, String>> entree = it.next ();
         Hashtable <String, String> elem = (Hashtable <String, String>) entree.getValue ();

         //System.err.println ("Working with " + entree.getKey ());

         if ((elem.get ("develops_from") != null) && (elem.get ("part_of") == null))
            {
             // Find the right organ for the part_of relation to add:

             // Note: we max have develops_from chains:
             // organ A <-(part_of) organ u <-(develops_from) organ t <-(develops_from) organ v
             String develops_from_parent_id = elem.get ("develops_from");
             String part_of_id = null;
             while (part_of_id == null)
                   {
                    // Find the hash corresponding to the parent by the develops_from relation:
                    Hashtable <String, String> parent_data = _triplets.get (develops_from_parent_id);
                    if (parent_data.get ("part_of") != null)
                       {
                        part_of_id = parent_data.get ("part_of");
                        _triplets.get (elem.get ("id")).put ("part_of", part_of_id);
                        }
                    else
                       if (parent_data.get ("is_a") != null)
                          {
                        part_of_id = parent_data.get ("is_a");
                        _triplets.get (elem.get ("id")).put ("is_a", part_of_id);
                           }
                     else
                       if (parent_data.get ("develops_from") != null)
                          {
                           develops_from_parent_id = parent_data.get ("develops_from");
                           }
                       else
                          {
                           // What shall we do?
                           System.err.println ("\tNothing found");
                           break;
                           }
                    }
             }
         }
  //System.err.println ("end of infer_part_of ()");
  }

 public Hashtable <String, Vector <String>> getCompleteDescritions ()
 { return _complete_descriptions;}
 }
