package datalayer;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.nio.channels.FileChannel;
import java.util.Vector;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import ontology.HomologyOntology;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class WorkspaceWriter
{
 String _base_dir;

 public WorkspaceWriter (String base_dir)
 {
  _base_dir = base_dir;
  }

 public void write (HomologyOntology ho)
 {
  DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
  DocumentBuilder constructeur;
File dir = new File (_base_dir);
dir.mkdir ();
  try
      {
       constructeur = fabrique.newDocumentBuilder();

       Document document = constructeur.newDocument();

       // Proprits du DOM
       document.setXmlVersion ("1.0");
       document.setXmlStandalone (true);

       // Cration de l'arborescence du DOM
       Element racine = document.createElement ("workspace");
       //racine.appendChild (document.createComment(""));

       Element eho = document.createElement("homology_ontology");
       eho.setAttribute("uri", "homology.obo");
       racine.appendChild (eho);

       // Add the list of species ontology files:
       Vector <String> su = ho.getSpeciesURIs ();
       System.err.println ("Number of species to add: " + su.size ());
       for (int i = 0; i < su.size (); i++)
           {
            Element species1 = document.createElement ("species");
            species1.setAttribute ("uri", _base_dir + File.separator + new File (su.get (i)).getName ());
            eho.appendChild (species1);
            // Copy the corresponding file to the right place:
            FileChannel source = null;
            FileChannel dest = null;
            try
                {
                 source = new FileInputStream (su.get (i)).getChannel ();
                 dest = new FileOutputStream (_base_dir + File.separator + new File (su.get (i)).getName ()).getChannel ();
                 System.err.println ("Writing " + su.get (i) + " to " + _base_dir + File.separator + new File (su.get (i)).getName ());
                 source.transferTo (0, source.size(), dest);
                 }
            catch (IOException e) { e.printStackTrace (); }
            finally
                    {
                     if (source != null) { try { source.close (); } catch (IOException e) {} }
                     if (dest != null) { try { dest.close (); } catch (IOException e) {} }
                     }
            }

       document.appendChild (racine);

       // Create the file:
       Source source = new DOMSource(document);

       File file = new File (_base_dir + File.separator + "data.xml");
       Result resultat = new StreamResult (file);
       
       TransformerFactory fabrique2 = TransformerFactory.newInstance ();
       Transformer transformer = fabrique2.newTransformer ();
       transformer.setOutputProperty (OutputKeys.INDENT, "yes");
       transformer.setOutputProperty (OutputKeys.ENCODING, "ISO-8859-1");
       
       // Transformation
       transformer.transform (source, resultat);
       }
  catch (ParserConfigurationException e)
        {
         e.printStackTrace();
         }
  catch (TransformerException e)
        {
         e.printStackTrace();
         }
  }

 public void update ()
 {
  
  }

 public WorkspaceWriter (String filename, String disc, HomologyOntology ho, String sannot)
 {
  // Create a new DOM:
  DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
  DocumentBuilder constructeur;
 
 try
     {
      constructeur = fabrique.newDocumentBuilder();

      Document document = constructeur.newDocument();

      // Proprits du DOM
      document.setXmlVersion ("1.0");
      document.setXmlStandalone (true);

      // Cration de l'arborescence du DOM
      Element racine = document.createElement ("workspace");
      racine.setAttribute("uri", filename );
      //racine.appendChild (document.createComment(""));

      Element eho = document.createElement("homology_ontology");
      eho.setAttribute("uri", ho.getBaseURI ());
      racine.appendChild (eho);

      // Add the list of species ontology files:
      Vector <String> su = ho.getSpeciesURIs ();
      System.err.println ("Number of species to add: " + su.size ());
      for (int i = 0; i < su.size (); i++)
          {
           Element species1 = document.createElement ("species");
           species1.setAttribute ("uri", su.get (i));
           eho.appendChild (species1);
           }

      // Add discards file:
      Element discards = document.createElement ("discards");
      discards.setAttribute ("uri", (disc == null ? "" : disc));
      eho.appendChild (discards);

      // Add annotations file:
      Element annot = document.createElement ("annotations");
      annot.setAttribute ("uri", (sannot == null ? "" : sannot));
      eho.appendChild (annot);

      document.appendChild (racine);

      // Create the file:
      Source source = new DOMSource(document);

      File file = new File(filename);
      Result resultat = new StreamResult (file);
      
      TransformerFactory fabrique2 = TransformerFactory.newInstance ();
      Transformer transformer = fabrique2.newTransformer ();
      transformer.setOutputProperty (OutputKeys.INDENT, "yes");
      transformer.setOutputProperty (OutputKeys.ENCODING, "ISO-8859-1");
      
      // Transformation
      transformer.transform (source, resultat);
      }
 catch (ParserConfigurationException e)
       {
        e.printStackTrace();
        }
 catch (TransformerException e)
       {
        e.printStackTrace();
        }
  }

 public WorkspaceWriter (String workspace_filename, String _homology_ontology_filename, String discards_filename, HomologyOntology ho, String sannot)
 {
  // Create a new DOM:
  DocumentBuilderFactory fabrique = DocumentBuilderFactory.newInstance();
  DocumentBuilder constructeur;
    
    try
        {
         constructeur = fabrique.newDocumentBuilder();

         Document document = constructeur.newDocument();

         // Proprits du DOM
         document.setXmlVersion ("1.0");
         document.setXmlStandalone (true);

         // Cration de l'arborescence du DOM
         Element racine = document.createElement ("workspace");
         racine.setAttribute("uri", workspace_filename );
         //racine.appendChild (document.createComment(""));

         Element eho = document.createElement("homology_ontology");
         eho.setAttribute("uri", ho.getBaseURI ());
         racine.appendChild (eho);

         // Add the list of species ontology files:
         Vector <String> su = ho.getSpeciesURIs ();
         System.err.println ("Number of species to add: " + su.size ());
         for (int i = 0; i < su.size (); i++)
             {
              Element species1 = document.createElement ("species");
              species1.setAttribute ("uri", su.get (i));
              eho.appendChild (species1);
              }

         // Add discards file:
         Element discards = document.createElement ("discards");
         discards.setAttribute ("uri", (discards_filename == null ? "" : discards_filename));
         eho.appendChild (discards);

         // Add annotations file:
         Element annot = document.createElement ("annotations");
         annot.setAttribute ("uri", (sannot == null ? "" : sannot));
         eho.appendChild (annot);

         document.appendChild (racine);

         // Create the file:
         Source source = new DOMSource(document);

         File file = new File (workspace_filename);
         Result resultat = new StreamResult (file);
         
         TransformerFactory fabrique2 = TransformerFactory.newInstance ();
         Transformer transformer = fabrique2.newTransformer ();
         transformer.setOutputProperty (OutputKeys.INDENT, "yes");
         transformer.setOutputProperty (OutputKeys.ENCODING, "ISO-8859-1");
         
         // Transformation
         transformer.transform (source, resultat);
         }
    catch (ParserConfigurationException e)
          {
           e.printStackTrace();
           }
    catch (TransformerException e)
          {
           e.printStackTrace();
           }
  }

 /**
  * 
  * @param wr
  * @param wr2
  * @param result
  */
 public WorkspaceWriter(String f, WorkspaceReader wr, WorkspaceReader wr2, HomologyOntology result)
 {
  // Create a new DOM:
  DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
  DocumentBuilder builder;
  _base_dir = f;

  // Create the directory:
  File dir = new File (_base_dir);
  dir.mkdir ();

  OboHomologyWriter ohw = new OboHomologyWriter (f + File.separator + "homology.obo", result);

  try
      {
       builder = factory.newDocumentBuilder();

       Document document = builder.newDocument();

       // DOM property:
       document.setXmlVersion ("1.0");
       document.setXmlStandalone (true);

       // XML tree creation:
       Element root = document.createElement ("workspace");

	   Element eho = document.createElement("homology_ontology");
	   eho.setAttribute("uri", "homology.obo");
	   root.appendChild (eho);

	   // Add the list of species ontology files:
       Vector <String> su = wr.getSpeciesFilename();
       System.err.println ("Number of species to add: " + su.size ());
       for (int i = 0; i < su.size (); i++)
           {
            Element species1 = document.createElement ("species");
            species1.setAttribute ("uri", su.get (i));
            eho.appendChild (species1);
            // Copy the corresponding file to the right place:
            FileChannel source = null;
            FileChannel dest = null;
            try
                {
                 source = new FileInputStream (wr.getBaseDirectory () + File.separator + new File (su.get (i)).getName ()).getChannel ();
                 dest = new FileOutputStream (_base_dir + File.separator + new File (su.get (i)).getName ()).getChannel ();
                 source.transferTo (0, source.size(), dest);
                 }
            catch (IOException e) { e.printStackTrace (); }
            finally
                    {
                     if (source != null) { try { source.close (); } catch (IOException e) {} }
                     if (dest != null) { try { dest.close (); } catch (IOException e) {} }
                     }
            }

       document.appendChild (root);

       // Create the file:
       Source source = new DOMSource(document);

       File file = new File (_base_dir + File.separator + "data.xml");
       Result resultat = new StreamResult (file);
       
       TransformerFactory fabrique2 = TransformerFactory.newInstance ();
       Transformer transformer = fabrique2.newTransformer ();
       transformer.setOutputProperty (OutputKeys.INDENT, "yes");
       transformer.setOutputProperty (OutputKeys.ENCODING, "ISO-8859-1");
       
       // Transformation
       transformer.transform (source, resultat);
	         }
	    catch (ParserConfigurationException e)
	          {
	           e.printStackTrace();
	           }
	    catch (TransformerException e)
	          {
	           e.printStackTrace();
	           }
  }
 }