package ontology;

import java.io.*;
import java.util.Vector;

public class HomologyOntology extends OboOntology implements Serializable
{
 /**
  * Id of the homology class.
  */
 String _hcId;

 /**
  * Id of the name property used in an homology object.
  */
 String _npId;

 /**
  * Used to keep trace of the last id used. This should be replaced by a smarter system to
  * generate ids.
  */
 int _hog_id;

 /**
  * The list of URIs for the species that are addressed in this homology ontology.
  * Currently it is filemanes.
  */
 Vector <String> _species_uris;
 Vector <String> _so_names;
 
 Vector <SpeciesOntology> _species_ontologies;

 private static final long serialVersionUID = 1L;

 public HomologyOntology (String name)
 {
  super (name);
  _species_ontologies = new Vector <SpeciesOntology> ();
  _so_names = new Vector <String> ();

  _species_uris = new Vector <String> ();

  _hcId = "OGE:0000100";
  OntologyClass ocHomologyGroup = new OntologyClass ("HomologyGroup", _hcId);
  OntologyClass ocOrgan = new OntologyClass ("Organ", "Organ");
  _npId = "internal:name";
  OntologyProperty ocName = new OntologyProperty ("name", _npId, ocHomologyGroup.getId (), "literal:String");
  setupPartOfProperty (_hcId, "Organ");
  addClass (ocHomologyGroup);
  addClass (ocOrgan);
  addProperty (ocName);
  addProperty (_opPartOf);

  _hog_id = 0;
  }

 /**
  * This constructor is used to merge two existing homology ontologies.
  * @param ho1
  * @param ho2
  */
 public HomologyOntology (HomologyOntology ho1, HomologyOntology ho2)
 {
  super ("merge");
  System.err.println (ho1.getBaseURI() + " " + ho2.getBaseURI());

  _hcId = "OGE:0000100";
  OntologyClass ocHomologyGroup = new OntologyClass ("HomologyGroup", _hcId);
  OntologyClass ocOrgan = new OntologyClass ("Organ", "Organ");
  _npId = "internal:name";
  OntologyProperty ocName = new OntologyProperty ("name", _npId, ocHomologyGroup.getId (), "literal:String");
  setupPartOfProperty (_hcId, "Organ");
  addClass (ocHomologyGroup);
  addClass (ocOrgan);
  addProperty (ocName);
  addProperty (_opPartOf);

  _hog_id = ho1._hog_id + 1;
  
  _instances = ho1._instances;
  for (int i = 0; i < ho2._instances.size (); i++)
      {
       OntologyInstance cg = ho2._instances.get (i);
       if (cg.getOntologyClassId ().equals (_hcId))
          {
           Vector <String> v = cg.getObjectOfPropertyByPropertyIdAsVector ("inv_" + getPartOfPropertyId ());
           System.err.println ("Working with " + cg.getId () + ": " + v);
           // Find all groups that contain one of the members of cg:
           Vector <OntologyInstance> to_merge = new Vector <OntologyInstance> ();
           for (int j = 0; j < v.size (); j++)
               {
        	    System.err.println ("\tInstance: " + v.get (j));
                for (int k = 0; k < _instances.size (); k++)
                 if (_instances.get (k).getOntologyClassId ().equals (this.getHomologyGroupClassId ()))
                    {
                	 Vector <String> vis = _instances.get(k).getObjectOfPropertyByPropertyIdAsVector ("inv_" + _poId);
                     System.err.println ("\t=>Checking in: " + _instances.get (k).getId () + ": " + vis);
                     if (vis.contains (v.get (j)))
                        {
                         to_merge.add (_instances.get (k));
                         break;
                         }
                     }
                }

           System.err.println ("to_merge: " + to_merge.size ());

           if (to_merge.size () > 0)
              {
        	   System.err.println ("Merging...");
               OntologyInstance hg = new OntologyInstance (this, cg.getName (), getNextHomologyGroupId (), this.getHomologyGroupClassId (), new Vector <String> (), null);
               this.addInstance (hg);

               // Add current group:
               for (int j = 0; j < v.size (); j++)
                   {
                    String cid = v.get (j);
                    hg.addProperty ("inv_" + this.getPartOfPropertyId (), cid);
                    // Retrieve the instance in ho2._instances and add it to this._instances:
                    for (int k = 0; k < ho2._instances.size (); k++)
                        {
                         OntologyInstance coi = ho2._instances.get (k);
                         if (coi.getId ().equals (cid))
                            {
                             coi.flush_property (_poId);
                             coi.addProperty (this.getPartOfPropertyId (), hg.getId ());
                             _instances.add (coi);
                             break;
                             }
                         }
                    }

               // Add the merged groups:
               for (int j = 0; j < to_merge.size (); j++)
                   {
                    Vector <String> vis = to_merge.get(j).getObjectOfPropertyByPropertyIdAsVector ("inv_" + _poId);
                    for (int k = 0; k < vis.size (); k++)
                        {
                    	System.err.println ("vis: " + vis);
                         for (int l = 0; l < _instances.size (); l++)
                             {
                              OntologyInstance coi = _instances.get (l);
                              System.err.println ("Testing: " + coi.getId());
                              if (coi.getId ().equals (vis.get(k)) && ! (hg.hasInstanceAsSubjectOfProperty (coi, "inv_" + this.getPartOfPropertyId ())))
                                 {
                            	  System.err.println ("Adding: " + coi.getId());
                                  coi.flush_property (_poId);
                                  coi.addProperty (this.getPartOfPropertyId (), hg.getId ());
                                  hg.addProperty ("inv_" + this.getPartOfPropertyId (), coi.getId());
                                  break;
                                  }
                              }
                         }
                    // Remove the old group from _instances:
                    _instances.remove (to_merge.get (j));
                    }
               }
           else
              {
               // Create a new homology group based on cg:
               OntologyInstance hg = new OntologyInstance (this, cg.getName (), getNextHomologyGroupId (), this.getHomologyGroupClassId (), new Vector <String> (), null);
               this.addInstance (hg);
               for (int j = 0; j < v.size (); j++)
                   {
                    String cid = v.get (j);
                    hg.addProperty ("inv_" + this.getPartOfPropertyId (), cid);
                    // Retrieve the instance in ho2._instances and add it to this._instances:
                    for (int k = 0; k < ho2._instances.size (); k++)
                        {
                         OntologyInstance coi = ho2._instances.get (k);
                         if (coi.getId ().equals (cid))
                            {
                             coi.flush_property (_poId);
                             coi.addProperty (this.getPartOfPropertyId (), hg.getId ());
                             _instances.add (coi);
                             break;
                             }
                         }
                    }
               }
           } 
       }
  }

 public void setHomologyGroupLastUsedId (int id) { _hog_id = id; }
 
 public String getHomologyGroupClassId () { return _hcId; }
 public String getNamePropertyId () { return _npId; }

 public Vector<OntologyInstance> getHomologyGroupsByName (String name)
 {
  Vector<OntologyInstance> res = new Vector<OntologyInstance> ();
  // Pour chaque groupe d'homologie la recherche doit se faire dans sa propriete name.
  for (int i = 0; i < _instances.size (); i++)
      {
       OntologyInstance oi = (OntologyInstance) _instances.get (i);
       Vector <String> v = oi.getObjectOfPropertyByPropertyIdAsVector (_npId);
       if (v != null)
          {
           for (int j = 0; j < v.size (); j++)
               if (((String) v.elementAt (j)).equals (name))
                  {
                   res.add (oi);
                   break;
                   }
           }
       }
  return res;
  }

 public void addSpeciesOntology (SpeciesOntology so)
 {
  _species_ontologies.add (so);
  _so_names.add (so.getBaseURI ());
  }

 public void setupPartOfProperty (String subjectClassId, String objectClassId)
 {
  _opPartOf = new OntologyProperty (_poName, _poId, subjectClassId, objectClassId);
  }

 /**
  * 
  * @param uri
  */
 public void AddSpeciesURI (String uri)
 {
  System.err.println ("Adding uri for species: " + uri);
  _species_uris.add (uri);
  }

 /**
  * 
  * @return
  */
 public Vector <String> getSpeciesURIs ()
 {
  return _species_uris;
  }

 /**
  * 
  * @return
  */
 public String getNextHomologyGroupId ()
 {
  String s_id = new Integer (_hog_id++).toString ();
  String source_id = "HOG:";
  for (int i = 0; i < 7 - s_id.length (); i++)
      source_id += "0";
  source_id += s_id;
  return source_id;
  }

 /**
  * Check in all existing honology groups if one of them contains the organ given as a
  * parameter.
  * @param oi The organ we are looking for.
  */
 public OntologyInstance getHomologyGroupOfOrgan (OntologyInstance oi)
 {
  // For each hoomology group, check if we have a part_of property with the given organ
  // as subject. if it is the case we can return this group, as an organ belongs to one
  // and only one group.
  for (int i = 0; i < _instances.size (); i++)
      {
       OntologyInstance hog = (OntologyInstance) _instances.get (i);
       if (hog.hasInstanceAsSubjectOfProperty (oi, "inv_" + _poId))
          return hog;
       }
  return null;
  }

 /**
  * Merges the two homology groups given as arguments. <code>oi2</code> is added to
  * <code>oi1</code>.
  * @param oi1 The first homology group, this one will contain the result of the merging
  * process.
  * @param oi2 The second homology group. This one will disappear.
  */
 public void merge (OntologyInstance oi1, OntologyInstance oi2)
 {
  // Lister les organes inclus dans oi2. Ceux qui ne sont pas presents dans oi1 doivent
  // etre ajoutes.
  Vector <String> ois = oi2.getObjectOfPropertyByPropertyIdAsVector (getPartOfPropertyId ());
  for (int i = 0; i < ois.size (); i++)
      {
       String org_id = (String) ois.get (i);

       // Ici ca va pas, il me faut l'ontologie de reference ou l'ontologie de travail, je
       // ne peux meme pas determiner a quelle ontologie appartient l'organe en question
       // (org_id).
       if (! oi1.hasInstanceAsSubjectOfProperty (getInstanceById (org_id), "inv_" + _poId))
          oi1.addProperty (getPartOfPropertyId (), org_id);
       }
       // /!\ TODO : Suppprimer oi2 de la liste des groupes d'homologie.
  }

 /**
  * 
  * @param id
  * @param names
  * @param members
  */
 public void addInstanceFromRawData (String id, Vector <String> names, Vector <String> members)
 {
  // TODO: add the other names.
  OntologyInstance hg = new OntologyInstance (this, names.get (0), id, this.getHomologyGroupClassId (), new Vector <String> (), null);
  this.addInstance (hg);

  // Add the instances to the instances of this ontology. This is done to ease the
  // display of the ontology of homology group.
  for (int i = 0; i < members.size (); i++)
      {
       hg.addProperty ("inv_" + this.getPartOfPropertyId (), members.get (i));
       OntologyInstance oi = new OntologyInstance (null, "", members.get (i), "", new Vector <String> (), null);
       oi.addProperty (this.getPartOfPropertyId (), hg.getId ());
                this.addInstance (oi);
       }

  // Maintain _hog_id:
  String [] words = id.split (":");
  System.err.println ("Split: " + words [0] + " and " + words [1]);
  int iid = new Integer (words [1]).intValue ();
  if (iid > _hog_id) _hog_id = iid;
  }

 public Vector <String> getOntologyFilenames () { return _so_names; }
 }