package ontology;

import java.io.*;
import java.util.LinkedList;
import java.util.List;
import java.util.Vector;

// /!\ TODO LIST:
// - Dans la fonction InsertSortedList j'utilise la fonction toString sur mes objects et
//   je ne pense pas que ce soit une bonne idee. En effet, cette fonction retournera le
//   nom de l'objet stocke, et il faut faire le tri sur l'id de l'objet, celui-ci etant
//   unique contrairement au nom.
//   Ca impose de modifier cette fonction pour qu'elle insere explicitement des objets
//   d'un type predefini.

/**
 * Most general class to represent an ontology. We store information about classes,
 * properties and instances of classes.
 * We also implement some mechanism to navigate faster in the ontology, like the
 * <code>_inverseProperties</code> vector.
 *
 * @author Gilles Parmentier
 */

public class Ontology implements Serializable
{
 protected List<OntologyInstance> _instances;
 protected Vector<OntologyClass> _classes;
 protected Vector<OntologyProperty> _properties;

 /**
  * Base uri of this ontology. Currently it is the complete pathname of the file used to store it.
  */
 String _base_uri = null;

 private static final long serialVersionUID = 1L;

 /**
  * Stores the complementary of the properties defined in the ontology. For example, if we
  * have a property X, with A as the subject and B as the object, in the inverse property
  * vector of B we will store A. The property vector of A receiving B as usual.
  */
 protected Vector <OntologyProperty> _inversedProperties;
 protected String _name;
 protected Vector<String> _imports; // Stores the name of the ontology that are imported.

 public Ontology (String n)
 {
  _name = n;
  _classes = new Vector<OntologyClass> ();
  _instances = new LinkedList<OntologyInstance> ();
  _properties = new Vector<OntologyProperty> ();
  _imports = new Vector<String> ();
  }

 // Les fonctions d'ajout a l'ontologie :

 void addClass (OntologyClass c) { _classes.add (c); /* System.out.println ("Adding " + c.toString ()); */ }

 public void addInstance (OntologyInstance i)
 {
  if (_instances.size () == 0) { _instances.add (i); return; }
  insertSortedList (_instances, 0, _instances.size () - 1, i);
  //System.out.println ("Adding " + i.toString ());
  }

 void insertSortedList (List<OntologyInstance> l, int lb, int ub, Object object)
 {
  OntologyInstance o = (OntologyInstance) object;
  if (ub == lb)
     {
      if ( o.toString ().compareTo (l.get (lb).getId ()) < 0)
         {
     //System.out.println ("On ajoute " + o.toString () + " avant " + l.get (lb).toString ());
          l.add (lb, o);
          }
      else
         {
     //System.out.println ("On ajoute " + o.toString () + " apres " + l.get (lb).toString ());
          l.add (lb + 1, o);
          }
      return;
      }
  int middle = lb + (ub - lb) / 2;
  if (middle <= lb)
     {
      // J'ai la flemme de verifier si middle peut etre inferieur a lb...
      middle = lb;
      // On insere juste avant ou juste apres middle
      if (o.getId ().compareTo (l.get (middle).getId ()) < 0)
         {
          // System.out.println ("On ajoute " + o.toString () + " avant " + l.get (middle).toString ());
          l.add (middle, o);
          }
      else
         if (o.getId ().compareTo (l.get (ub).getId ()) < 0)
            {
//             System.out.println ("On ajoute " + o.toString () + " apres " + l.get (middle).toString ());
             l.add (middle + 1, o);
             }
         else
            {
//             System.out.println ("On ajoute " + o.toString () + " apres " + l.get (ub).toString ());
             l.add (ub + 1, o);
             }
      return;
      }
  else
     if (o.getId ().compareTo (l.get (middle).getId ()) < 0)
        insertSortedList (l, lb, middle, o);
     else
        insertSortedList (l, middle, ub, o);
  }

 void addProperty (OntologyProperty p) { _properties.add (p); }

 OntologyProperty getProperty (String id)
 {
  for (int i = 0; i < _properties.size (); i++)
      {
       OntologyProperty op = _properties.elementAt (i);
       if (op.getId ().equals (id))
          return op;
       }
  return null;
  }

 void addImport (String name) { _imports.add (name); }

 public List <OntologyInstance> getInstances () { return _instances; } // Je sais pas ou j'utilise cette fonction mais il faudra la degager, c'est crado... /!\ TODO

 public LinkedList <OntologyInstance> getInstancesAsLinkedList ()
 {
  //System.out.println ("Requiring intances as a linked list...");
  return (LinkedList<OntologyInstance>) _instances;
  }

 // Returns all the instance whose _name is equal to the given argument.
 public Vector<OntologyInstance> getInstancesByNameAsVector (String name)
 {
  Vector<OntologyInstance> res = new Vector<OntologyInstance> ();
  for (int i = 0; i < _instances.size (); i++)
      if (_instances.get (i).getName ().equals (name))
         res.add (_instances.get (i));
  return res;
  }

 public Vector <OntologyInstance> getInstancesBySynonymAsVector (String name)
 {
  Vector <OntologyInstance> res = new Vector <OntologyInstance> ();
  for (int i = 0; i < _instances.size (); i++)
  {
  if (_instances.get (i).hasSynonym (name))
      res.add (_instances.get (i));
  }
  return res;
  }

 /**
  * Creates a property that is intended to be the inverse of an existed one. We only
  * create the <code>OntologyProperty</code> object here.
  * The id and the name of the new property will be created by adding the prefix "inv_" to
  * the id and name of the original property.
  * @param p The id of the property for which we want to create an inversed one.
  * @return The created property.
  */

 public OntologyProperty addInverseProperty (String p)
 {
  OntologyProperty op = getProperty (p);
  // The order of the object and subject type should be reversed compared to the original property:
  OntologyProperty iop = new OntologyProperty ("inv_" + op.getName (), "inv_" + op.getId (), op.getSubjectClass (), op.getObjectClass ());
  return iop;
  }

 /**
  * @param id The id of the <code>OntologyInstance</code> we are looking for.
  * @return The wanted OntologyInstance if it has been found, null otherwise.
  */

 public OntologyInstance getInstanceById (String id)
 {
  for (int i = 0; i < _instances.size (); i++)
      if (_instances.get (i).getId ().equals (id))
         return _instances.get (i);
  return null;
  }

 public String toString () { return "An ontology"; }

 // Getters:

 public Vector <OntologyClass> getClasses () { return _classes; }

 public Vector<OntologyProperty> getProperties () { return _properties; }

 public OntologyProperty getPropertyById (String id)
 {
  for (int i = 0; i < _properties.size (); i++)
      if (_properties.get (i).getId ().equals (id))
         return _properties.get (i);
  return null;
  }

 /**
  * 
  *
  */
 public void setBaseURI (String bu)
 {
  _base_uri = bu;
  }
 
 /**
  * 
  * @return
  */
 public String getBaseURI () { return _base_uri; }
 }