package ontology;

import java.io.*;
import java.util.Vector;

/**
 * A class to model an instance of an ontology class.
 *
 * @author Gilles Parmentier
 */

public class OntologyInstance extends OntologyObject implements Serializable
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;
 protected String _name;
 protected String _class;
 Vector <String> _complete_description;

 // _properties is a Vector of Vector. Each Vector contains objects for the corresponding property.
 // _propertiesType specifies the property name for each position in _properties.
 protected Vector <Vector <String> > _properties;
 protected Vector <String> _propertiesType;

 protected Ontology _myOntology; // The ontology this instance belongs to.

 public OntologyInstance (Ontology o, String name, String id, String clss, Vector <String> synonyms, Vector <String> desc)
 {
  super (id);
  _name = name;
  _class = clss;
  _properties = new Vector <Vector <String> > ();
  _propertiesType = new Vector <String> ();
  _myOntology = o;
  _complete_description = desc;

  if (synonyms.size () > 0)
  {
   for (int i = 0; i < synonyms.size (); i++)
       addProperty ("synonym", synonyms.get (i));
   }
  }

 public String toString () { return _id; }

 public String getName () { return _name; }

 /**
  * This function adds a property to the current instance. It means that this instance is
  * the subject of the property and that the argument is the object of the property.
  *
  * @param property_id The id of the property that is added to this instance.
  * @param jeobjectct The id of the object of the property.
  */

 public void addProperty (String property_id, String jeobjectct)
 {
  // Verifier si on a deja stocke une propriete de ce type :
  int good_pos = - 1;
  for (int i = 0; i < _propertiesType.size (); i++)
      if (property_id.equals (_propertiesType.elementAt (i).toString ())) good_pos = i;

  if (good_pos == - 1)
     {
      //System.out.println ("\t Creating new property " + property_id + " (" + this._id + "):" + jeobjectct);
      // Create a new property Vector:
      Vector<String> new_property = new Vector<String> ();
      new_property.add (jeobjectct);
      _properties.add (new_property);
      _propertiesType.add (property_id);
      }
  else
     {
      //System.out.println ("\t Adding property " + property_id + " (" + this._id + "):" + jeobjectct);
      // Complete the existing property Vector:
      Vector<String> vProp = _properties.elementAt (good_pos);
 vProp.add (jeobjectct);
      }
  }

 public void flush_property (String prop)
 {

 for (int i = 0; i < _propertiesType.size (); i++)
     if (prop.equals (_propertiesType.elementAt (i).toString ()))
        {
         _properties.set (i, new Vector <String> ());
         }
  }

 public Vector <String> getBrothersInProperty (String property_id)
 {
  Vector<String> res = new Vector<String> ();
  // Premierement je dois retrouver son parent pour la propriete consideree :
  int good_pos = - 1;
  for (int i = 0; i < _propertiesType.size (); i++)
      if (property_id.equals (_propertiesType.elementAt (i).toString ())) good_pos = i;
  if (good_pos != - 1)
     {
      Vector<String> vProp = _properties.elementAt (good_pos);
      for (int i = 0; i < vProp.size (); i++)
          {
           String father = vProp.get (i);
           System.out.println ("One father: " + father);
// Maintenant il faut que je trouve tous les noeuds de son ontologie qui ont father pour pere...
           OntologyInstance oifather = _myOntology.getInstanceById (father);
           Vector <String> v = oifather.getObjectOfPropertyByPropertyIdAsVector("inv_" + property_id);
           for (int j = 0; j < v.size (); j++)
 if (! ((String) v.get(j)).equals(_id))
 {
System.out.println ("Adding a brother: " + (String) v.get (j));
	 res.add ((String) v.get(j));
 }
           }
      }
  return res;
  }

 /**
  * Returns the object of the property given as an argument, for wich the current
  * instance is the subject.
  * @param property_id The id of the property for which we want the objects of its
  * instances having the current instance as the subject.
  * @return A <code>Vector</code> of elements that are objects for the given property,
  * with the current instance as subject.
  */
 // Fonction qui renvoie tous les objets d'une propriete dont le nom est donne en argument. Le sujet de la propriete etant l'instance courante.
 public Vector <String> getObjectOfPropertyByPropertyIdAsVector (String property_id)
 {
  //System.out.println ("Looking for property: " + property_id + " (" + _id + ")");
  // Trouver la position du vector correspondant a notre property :
  int good_pos = -1;
  for (int i = 0; i < _propertiesType.size (); i++)
      {
       //System.out.println ("Checking: " + (String) _propertiesType.elementAt (i));
       if (_propertiesType.elementAt (i).equals (property_id)) good_pos = i;
       }
  if (good_pos == - 1) return null; // No property corresponding to the given id.
  return _properties.elementAt (good_pos);
  }

 // Debugging functions:

 public void printProperties ()
 {
  for (int i = 0; i < _propertiesType.size (); i++)
      System.out.println (_propertiesType.get (i));
  }

 public void displayAsTreeText (String dec)
 {
  //  System.out.println (dec + _id + ": " + _name);
  Vector <String> children = getObjectOfPropertyByPropertyIdAsVector ("inv_" + ((SpeciesOntology) _myOntology).getPartOfPropertyId ());
  if (children != null)
     {
      for (int i = 0; i < children.size (); i++)
          {
           String child_id = (String) children.get (i);
           OntologyInstance child = _myOntology.getInstanceById (child_id);
           child.displayAsTreeText (dec + "\t");
           }
      }
  }

 // Getters:
 public Ontology getOntology () { return _myOntology; }

 // Testers:
 public boolean hasInstanceAsSubjectOfProperty (OntologyInstance oi, String prop_id)
 {
  //System.out.println ("Checkinf if " + oi.getId () + " " + prop_id + " " + this.getId ());
  for (int i = 0; i < _propertiesType.size (); i++)
      {
       if (_propertiesType.get (i).equals (prop_id))
          {
           Vector <String> properties = _properties.get (i);
           //System.out.println ("property found " + properties.size ());
           for (int j = 0; j < properties.size (); j++)
               {
                //System.out.println ("Checking " + (String) properties.get (j));
                if (((String) properties.get (j)).equals (oi.getId ()))
                   {
                    //System.out.println ("It is.");
                    return true;
                    }
                }
           break;
           }
       }
  return false;
  }

 public Object getOntologyClassId ()
 {
 return _class;
 }

public boolean hasSynonym (String name)
{
 for (int i = 0; i < _propertiesType.size (); i++)
     if (_propertiesType.get (i).equals ("synonym"))
        {
         Vector <String> synonyms = _properties.get (i);
         for (int j = 0; j < synonyms.size (); j++)
             if (name.equals (synonyms.get (j)))
                return true;
         }
 return false;
 }

public Vector <String> getFullDescription ()
{
 return this._complete_description;
 }
 }