package visualization;

import java.awt.BorderLayout;
import java.awt.CardLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.event.*;
import java.awt.geom.Line2D;
import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.ButtonGroup;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.event.HyperlinkEvent;
import javax.swing.event.HyperlinkListener;
import javax.swing.text.html.HTMLEditorKit;
import alignment.AnnotatedOrgan;
import alignment.HomologyMaker;
import alignment.OrganHierarchyPairingScorer;
import alignment.OrganSimplePairingScorer;
import alignment.PairingScore;
import alignment.PairingScorer;
import alignment.SyntaxicPairingScorer;

import ontology.Annotations;
import ontology.Ontology;
import ontology.OntologyInstance;

class AlignmentFrame extends JFrame implements HyperlinkListener
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;
 JButton _jbAlign, _jbNext, _jbPrevious;
 JPanel _jpPropositions;
 JScrollPane _propView;
 CardLayout _cl;
 PairingScorer _scorer = null;
 Vector <Annotations> _annotations;
 HomologyMaker _aligner;
 String _name = "";
 GraphDialog _gd;

 Vector <UserDecision> _decisions;

 boolean _hasNewHomology = false;
 
 int _nb_validate, _nb_invalidate;

 /**
  * A <code>Vector</code> containing all the pairs of <code>AnnotatedOrgan</code> to be
  * reviewed by the user .
  */
 Vector <AnnotatedOrgan> _aoPropositions;

 /**
  * An array of <code>PairingScore</code>. This one exists in order to avoid the usage of
  * the <code>getBestScoreObject ()</code> function when we retrieve the choices of the
  * user.
  */
 PairingScore [] _psPropositions;

 int _current_pos; // Curseur qui indique quelle proposition on est en train d'afficher.
                   // Ca me permet de gerer les boutons next et Previous.
 JScrollPane _jsp;
 AnnotatedOrganPanel [] _aop;
 JPanel mainPanel;

 JButton _jbIterate;

 Main _m;

 Vector <String []> _list;

 /**
  * 
  * @param aligner
  * @param m
  */

 AlignmentFrame (HomologyMaker aligner, Main m)
 {
  super ();
  _list = new Vector <String []> ();
  _m = m;
  _aligner = aligner;
  _decisions = new Vector <UserDecision> ();

  mainPanel = new JPanel (new BorderLayout ());
  _cl = new CardLayout ();
  _jpPropositions = new JPanel (_cl);
  _jsp = new JScrollPane (_jpPropositions);
  Dimension dim = new Dimension (800, 700);
  _jsp.setMinimumSize (dim);
  _jsp.setPreferredSize (dim);
  mainPanel.add (_jsp, BorderLayout.CENTER);

  JPanel controlButtons = new JPanel ();

  _jbIterate = new JButton (new IterateAction ("Iterate"));
  controlButtons.add (_jbIterate);

  _jbPrevious = new JButton (new PreviousPairingAction ("Previous"));
  _jbPrevious.setEnabled (false);
  controlButtons.add (_jbPrevious);

  _jbNext = new JButton (new NextAction ("Next"));
  _jbNext.setEnabled (false);
  controlButtons.add (_jbNext);

  mainPanel.add (controlButtons, BorderLayout.PAGE_END);
  setContentPane (mainPanel);
  pack ();

  addWindowFocusListener (new WindowAdapter() {
	    public void windowGainedFocus(WindowEvent e) {
	        _jbIterate.requestFocusInWindow();
	    }
	});

  _gd = new GraphDialog (this);
  _gd.setVisible (true);
  }

 /**
  * 
  * @param ao
  * @return
  */

 private Annotations findAnnotations (AnnotatedOrgan ao)
 {
  if (_annotations == null) return null;
  //System.err.println ("Size of annotations is: " + _annotations.size ());
  PairingScore ps = ao.getBestScoreObject ();
  if (ps == null) System.err.println ("Error: a pairing score is null");
  OntologyInstance oi = ao.getOrgan ();
  OntologyInstance oiCandidate = ps.getOrganCandidate ();
  for (int i = 0; i < _annotations.size (); i++)
      {
       //System.err.println ("Checking: " +_annotations.get (i).getWorker ().getId () + "("
       //+ oi.getId () + ")"  +" and " + _annotations.get (i).getReference ().getId () + "("
        //                                                    + oiCandidate.getId () + ")");
 Annotations annot = _annotations.get (i);
 OntologyInstance worker = annot.getWorker ();
 String local_id = oi.getId ();
 if (worker == null) System.err.println ("The working ontology OntologyInstance is null !");
       if (worker.getId ().equals (local_id) &&
              _annotations.get (i).getReference ().getId ().equals (oiCandidate.getId ()))
      {
   System.err.println ("Annotaions found");
         return _annotations.get (i);
      }
      }
  System.err.println ("No annotation found");
  return null;
  }

 /**
  * 
  *
  */
 void align ()
 {
  final BlockingDialog bf = new BlockingDialog (this, "Aligning...", "");
  SwingUtilities.invokeLater (new Runnable () { public void run () {
  bf.setVisible (true);
  }});
  _annotations = _aligner.getAnnotations ();

  long start = System.currentTimeMillis ();
  _aligner.align (_scorer);
  long end = System.currentTimeMillis ();
  long time = (end - start) / 1000;
  System.err.println ("total alignment time: " + time);

  SwingUtilities.invokeLater (new Runnable () { public void run () {
	  bf.setVisible (false);
  }});
 
 }

 public void setValidatedList (Vector <String []> l)
 {
  _list = l;
  }

 /**
  * Automatically validates the pair that are present in <param>list</param>.
  * @param list A <code>Vector</code> of pair of string that are validated homology pairs.
  */
 public void validateFromList ()
 {
  _aoPropositions = _aligner.getPropositions ();
  Vector <PairingScore> validated_strict = new Vector <PairingScore> ();
  for (int i = 0; i < _aoPropositions.size (); i++)
      {
       AnnotatedOrgan ao = _aoPropositions.get (i);
       // Check if the current proposition is in the validated list:
       for (int j = 0; j < _list.size (); j++)
           {
            String org_id = ao.getOrgan ().getId ();
            String s1 = _list.get (j) [1];
            String s0 = _list.get (j) [0];
            //System.err.println ("First proposition element is: " + org_id + " (" + _list.get (j) [1] + ")");
            if (org_id.equals (s1))
               {
             PairingScore ps = ao.getBestScoreObject ();
             if (ps != null)
             {
             String cand_id = ps.getOrganCandidate ().getId ();
             if (cand_id.equals (s0))
             {
                // Validate the current proposition:
                validated_strict.add (ps);
                _gd.addData (ps.getScore ());
                _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.VALIDATE, ps.getIntrinsicScore (), 1));
             }
             }
                }
            }
       }
  _aligner.iterate (validated_strict, new Vector <PairingScore> (), new Vector <PairingScore> (), new Vector <PairingScore> ());
  System.err.println ("end of validateFromList");
  }

 public void displayResults ()
 {
  SwingUtilities.invokeLater (new Runnable () { public void run () {
	  _aoPropositions = _aligner.getPropositions ();
	  _aop = new AnnotatedOrganPanel [_aoPropositions.size ()];
	  _psPropositions = new PairingScore [_aoPropositions.size ()];
	  if (_aoPropositions.size () > 1)
	     _jbNext.setEnabled (true);

	  _current_pos = 0;
	  final AnnotatedOrgan aoCurrent = (AnnotatedOrgan) _aoPropositions.get (0);
	  AnnotatedOrganPanel.setAuthorName (_name);

	   _aop [_current_pos] = new AnnotatedOrganPanel (AlignmentFrame.this, aoCurrent, findAnnotations (aoCurrent));
	   _psPropositions [_current_pos] = aoCurrent.getBestScoreObject ();
	   _jpPropositions.add (aoCurrent.getOrgan ().getId (), _aop [_current_pos]);
	   _cl.invalidateLayout (_jpPropositions);
	   invalidate ();
	   validate();
	   mainPanel.getInputMap (JComponent.WHEN_IN_FOCUSED_WINDOW).put (KeyStroke.getKeyStroke ("ENTER"), "enter");
	   mainPanel.getActionMap ().put ("enter", new ValidateAction ("Validate"));
	   mainPanel.getInputMap (JComponent.WHEN_IN_FOCUSED_WINDOW).put (KeyStroke.getKeyStroke ("ESCAPE"), "escape");
	   mainPanel.getActionMap ().put ("escape", new InvalidateAction ("Invalidate"));
	   mainPanel.getInputMap (JComponent.WHEN_IN_FOCUSED_WINDOW).put (KeyStroke.getKeyStroke ("RIGHT"), "right");
	   mainPanel.getActionMap ().put ("right", new NextAction ("Next"));
	   mainPanel.getInputMap (JComponent.WHEN_IN_FOCUSED_WINDOW).put (KeyStroke.getKeyStroke ("LEFT"), "left");
	   mainPanel.getActionMap ().put ("left", new PreviousPairingAction ("Previous"));
   }});
	  invalidate ();
  }

 // Getters:
 public Vector <Annotations> getAnnotations () { return _annotations; }

 public void setScoringScheme (int value)
 {
  switch (value)
         {
          case Main.SIMPLE_MATCHING:
               _scorer = new OrganSimplePairingScorer ();
               break;
          case Main.SIMPLE_WITH_HIERARCHY:
               _scorer = new OrganHierarchyPairingScorer ();
               break;
          case Main.PARTIAL_MATCHING:
               _scorer = new SyntaxicPairingScorer ();
               break;
          }
  PairingScore.setScorer (_scorer);
  _aligner.setScorer (_scorer);
  }

 //Testers:
 public boolean hasDecisions ()
 {
  System.err.println ("nb decisions: " + _decisions.size ());
  return (_decisions.size () > 0);
  }

 /**
  * @author Gilles Parmentier
  *
  * An <code>AbstractAction</code> implemntation to handle the stepping through the list
  * of pairings panels. This one allows to step forward int he list, i.e. to step towards
  * panel that have not been display yet.
  */
 class NextAction extends AbstractAction
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  NextAction (String name)
  {
   super (name);
   }

  public void actionPerformed (ActionEvent arg0)
  {
   nextPairing ();
   }
  }

 void nextPairing ()
 {
  _current_pos++;
  if (_current_pos == _aoPropositions.size ())
     {
      _current_pos = 0;
      _cl.next (_jpPropositions);
      _cl.invalidateLayout (_jpPropositions);
      this.invalidate ();
      this.validate();
      _jpPropositions.revalidate ();
      }
  else
     {
      if (_aop [_current_pos] == null)
         {
          System.err.println ("Creating new panel, pos: " + _current_pos);
          AnnotatedOrgan aoCurrent = (AnnotatedOrgan) _aoPropositions.get (_current_pos);

          _aop [_current_pos] = new AnnotatedOrganPanel (this, aoCurrent, findAnnotations (aoCurrent));
          _jpPropositions.add (aoCurrent.getOrgan ().getId (), _aop [_current_pos]);
          _psPropositions [_current_pos] = aoCurrent.getBestScoreObject ();
          }
      _cl.next (_jpPropositions);
      _cl.invalidateLayout (_jpPropositions);
      this.invalidate ();
      this.validate();
      _jpPropositions.revalidate ();
      }
  _jbPrevious.setEnabled (true);
  }
 
 /**
  * 
  * @author Gilles Parmentier
  *
  */
 class PreviousPairingAction extends AbstractAction
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  public PreviousPairingAction (String name)
  {
   super (name);
   }

  public void actionPerformed (ActionEvent arg0)
  {
   //System.err.println (arg0);
   previousPairing ();
   }
  }

 /**
  * 
  *
  */
 public void previousPairing ()
 {
  // Using shortcuts we may have this function called even if we are at the begining of
  // the list. Just do nothing.
  if (_current_pos ==0) return;
  if (_current_pos == 1) _jbPrevious.setEnabled (false);
  --_current_pos;
  //System.err.println ("Previous, new pos: " + _current_pos);
  _cl.previous (_jpPropositions);
  _cl.invalidateLayout (_jpPropositions);
  this.invalidate ();
  _jpPropositions.revalidate ();
  }
 
 /**
  * 
  * @author admin
  */ 
 class ValidateAction extends AbstractAction
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  ValidateAction (String name)
  {
   super (name);
   }

  public void actionPerformed (ActionEvent arg0)
  {
   if (! _hasNewHomology)
      {
       _hasNewHomology = true;
       _m.setHasNewHomology ();
       }

   validateHomology ();
   iterate ();
   }
  }

 void validateHomology ()
 {
  _aop [_current_pos].validateHomology ();
  }

 class InvalidateAction extends AbstractAction
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  public InvalidateAction (String name)
  {
   super (name);
   }

  public void actionPerformed (ActionEvent arg0)
  {
   invalidateHomology ();
   iterate ();
   }
  }

 void invalidateHomology ()
 {
  _aop [_current_pos].invalidateHomology ();
  }

 class IterateAction extends AbstractAction
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  IterateAction (String name)
  {
   super (name);
   }

  public void actionPerformed (ActionEvent arg0)
  {
   //new Thread (new Runnable () { public void run () {
   iterate ();
   //}}).start ();
   }
  }

 /**
  * 
  *
  */
 void iterate ()
 {
  _jbPrevious.setEnabled (false);
  Vector <PairingScore> validated = new Vector <PairingScore> ();
  final Vector <PairingScore> validated_strict = new Vector <PairingScore> ();
  Vector <PairingScore> invalidated = new Vector <PairingScore> ();
  Vector <PairingScore> delay_ignore = new Vector <PairingScore> ();

  //System.err.println ("_aoPropositions.size: " + _aoPropositions.size ());

  for (int i = 0; i < _aoPropositions.size (); i++)
      if (_aop [i] != null)
         {
          // part dedicated to the statistics for the article:
          int status = 0;

          for (int j = 0; j < _list.size (); j++)
              {
               String org_id = _aoPropositions.get(i).getOrgan ().getId ();
               String s1 = _list.get (j) [1];
               String s0 = _list.get (j) [0];
               //System.err.println ("First proposition element is: " + org_id + " (" + _list.get (j) [1] + ")");
               if (org_id.equals (s1))
                  {
                   PairingScore ps = _aoPropositions.get(i).getBestScoreObject ();
                   if (ps != null)
                      {
                       String cand_id = ps.getOrganCandidate ().getId ();
                       if (cand_id.equals (s0))
                          {
                           // Validate the current proposition:
                           status = 1;
                           }
                       }
                   }
               }
          //end of article part

          AnnotatedOrganPanel aopCurrent = _aop [i];
          _cl.removeLayoutComponent (_aop [i]);
          AnnotatedOrgan aoCurrent = (AnnotatedOrgan) _aoPropositions.get (i);
          PairingScore ps = _psPropositions [i];

          if (aopCurrent.getStatus () == AnnotatedOrganPanel.VALIDATED)
             {
              _nb_validate += 1;
              if (! _hasNewHomology)
                 {
                  _hasNewHomology = true;
                  _m.setHasNewHomology ();
                  }
              //System.err.println ("Validating: " + oiOrgan.getId () + ", " +
              //                oiOrgan.getName () + " with " + oiCandidate.getId () +", " +
              //                                                    oiCandidate.getName ());
              _gd.addData (ps.getScore ());
              _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.PARTIAL_VALIDATE, ps.getIntrinsicScore (), status));
              validated.add (ps);
              }
          else
             if (aopCurrent.getStatus () == AnnotatedOrganPanel.VALIDATED_STRICT) 
                {
                 _nb_validate += 1;
                 if (! _hasNewHomology)
                    {
                     _hasNewHomology = true;
                     _m.setHasNewHomology ();
                     }
                 //System.err.println ("Validating: " + oiOrgan.getId () + ", " +
                 //             oiOrgan.getName () + " with " + oiCandidate.getId () +", " +
                 //                                                 oiCandidate.getName ());
                 _gd.addData (ps.getScore ());
                 _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.VALIDATE, ps.getIntrinsicScore (), status));
                 validated_strict.add (ps);
                 }
             else
                if (aopCurrent.getStatus () == AnnotatedOrganPanel.INVALIDATED)
                   {
                    _nb_invalidate += 1;
                    _gd.addData (- ps.getScore ());
                    _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.INVALIDATE, ps.getIntrinsicScore (), status));
                    invalidated.add (ps);
                    }
                else
                   if (aopCurrent.getStatus () == AnnotatedOrganPanel.DELAY_AND_IGNORE)
                      {
                       //System.err.println ("Ignoring: " + oiOrgan.getId () + ", " +
                       //       oiOrgan.getName () + " with " + oiCandidate.getId () +", " +
                       //                                           oiCandidate.getName ());
                       _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.DELAY, ps.getIntrinsicScore (), status));
                       delay_ignore.add (ps);
                       }
                   else
                    if (aopCurrent.getStatus () == AnnotatedOrganPanel.DELAYED)
                       {
                        //System.err.println ("Ignoring: " + oiOrgan.getId () + ", " +
                        //       oiOrgan.getName () + " with " + oiCandidate.getId () +", " +
                        //                                           oiCandidate.getName ());
                        _decisions.add (new UserDecision (ps, ps.getScore (), UserDecision.DELAY, ps.getIntrinsicScore (), status));
                        }

          // Annotations:
          Annotations annot = aopCurrent.getAnnotations ();
          int j = 0;
          boolean found = false;
          if (_annotations != null)
          for (; j < _annotations.size (); j++)
              {
               // The annotation element may be null : sometimes the user is stepping
               //through the list without validating anything. Not sure it is the problem though.
               String annot_ref_id = _annotations.get (j).getReference ().getId ();
               if ((annot_ref_id != null) && annot_ref_id.equals (aoCurrent.getOrgan ().getId ()))
                  {
                   // Reference organs match. Check the working ontology organs:
                   if ((_annotations.get (j).getWorker ().getId ().equals (aoCurrent.getBestScoreObject ().getOrganCandidate ().getId ())))
                      {
                       //System.err.println ("Annotation replaced");
                       _annotations.set (j, annot);
                       found = true;
                       }
                  }
               }
          if (! found)
             {
              //System.err.println ("Annotation added");
              if (_annotations == null)
              _annotations = new Vector <Annotations> ();
              _annotations.add (annot);
              }
          }
  // For each validated pair, we want to look for putative similar pairs that could be
  // validated in the mean time. One way is to validate pairs one after another, calling
  // _aligner.iterate () for each one.

  // New code:
  for (int i = 0; i < validated_strict.size (); i++)
      {
       final BlockingDialog bf = new BlockingDialog (this, "", "Looking for similar pairs");
       final Vector <PairingScore> current_list = new Vector <PairingScore> ();
       final PairingScore current_pair = validated_strict.get (i);
       current_list.add (current_pair);
       //new Thread (new Runnable () {
       //public void run () {
       SwingUtilities.invokeLater (new Runnable () { public void run () {
       bf.setVisible (true);
       }});
       //System.err.println ("");
       //SimilarPairFinder spf = new SimilarPairFinder (current_pair, _aoPropositions);
       //Vector <PairingScore> other_pairs = spf.getResults ();
       //System.err.println ("");
       SwingUtilities.invokeLater (new Runnable () { public void run () {
       bf.setVisible (false);
       bf.dispose ();
       }});
//       if (other_pairs.size () > 0)
//          {
//           // Ici je devrais creeer une fenetre qui demande de confirmer la validation de
//	   // chacune des paires presentes dans other_pairs.
//	   MultipleChoiceValidationDialog mcvd = new MultipleChoiceValidationDialog (this, other_pairs);
//	   mcvd.setVisible (true);
//
//       // Check who is validated, invalidated, delayed:
//       Vector <Integer> r = mcvd.getChoices ();
//       for (int ri = 0; ri < r.size (); ri++)
//           {
//            if (r.get (ri) == 0)
//            {
//             current_list.add (other_pairs.get (ri));
//             _gd.addData (other_pairs.get (ri).getScore ());
//             }
//            else
//             if (r.get (ri) == 1)
//             {}
//             else
//              if (r.get (ri) == 2)
//              {
//               _gd.addData (- other_pairs.get (ri).getScore ());
//               invalidated.add (other_pairs.get (ri));
//              }
//            }
//           //current_list.addAll (other_pairs);
//           }
       // Validate the current pair and the similar ones:
       final BlockingDialog bf2 = new BlockingDialog (this, "", "Validating...");
       SwingUtilities.invokeLater (new Runnable () { public void run () {
       bf2.setVisible (true);
       }});
       _aligner.iterate (current_list, validated, new Vector <PairingScore> (), new Vector <PairingScore> ());
       SwingUtilities.invokeLater (new Runnable () { public void run () {
       bf2.setVisible (false);
       bf2.dispose ();
       }});
       }
  // Iterate for what is left:
  if ((validated.size () > 0) ||(invalidated.size () > 0) || (delay_ignore.size () > 0))
     _aligner.iterate (new Vector <PairingScore> (), validated, invalidated, delay_ignore);
  // End of new code.

// Code specific for the article:
  //validateFromList ();
  // End of specific code

  _aoPropositions = _aligner.getPropositions ();
  _aop = new AnnotatedOrganPanel [_aoPropositions.size ()];
  _psPropositions = new PairingScore [_aoPropositions.size ()];
  if (_aoPropositions.size () > 1) _jbNext.setEnabled (true);
  _current_pos = 0;
  AnnotatedOrgan aoCurrent = (AnnotatedOrgan) _aoPropositions.get (0);
  _aop [_current_pos] = new AnnotatedOrganPanel (this, aoCurrent, findAnnotations (aoCurrent));
  _psPropositions [_current_pos] = aoCurrent.getBestScoreObject ();
  // Il faut vider _jpPropositions :
  _jpPropositions.removeAll ();
  _cl.invalidateLayout (_jpPropositions);
  _jpPropositions.add (aoCurrent.getOrgan ().getId (), _aop [_current_pos]);
  _jpPropositions.revalidate ();

  _aligner.setAnnotations (_annotations);
  System.err.println ("Validated: " + _nb_validate);
  System.err.println ("Invalidated: " + _nb_invalidate);
  }

 class GraphDialog extends JDialog
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;
  Graph _g;

  GraphDialog (JFrame parent)
  {
   super (parent, "", false);
   _g = new Graph ();
   getContentPane ().add (_g, BorderLayout.CENTER);
   pack ();
   }

  public void addData (double d)
  { _g.addData (d); }
  }

 class Graph extends Component
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;
  Vector <Double> _data;
  int max_score = 100;

  Graph ()
  {
   super ();
   _data = new Vector <Double> ();
   }

  public void addData (double d) { _data.add (d); repaint (); }

  public void paint (Graphics g)
  {
   super.paint (g);
   Graphics2D g2 = (Graphics2D) g;
   double width = this.getWidth ();
   double height = this.getHeight ();

   // Draw axis:
   g2.draw (new Line2D.Double (10, height / 2 - 10, width - 10, height / 2 - 10));
   g2.draw (new Line2D.Double (10, height - 10, 10, 10));

   double increment = (width - 20) / ((double) _data.size () + 1);
   for (int i = 0; i < _data.size (); i++)
       {
        if (_data.get (i) < 0)
           {
    	    g2.draw (new Line2D.Double (10 + increment * (i + 1), height / 2 - 10,
                   10 + increment * (i + 1),
                  ((- _data.get(i)) > max_score ? height - 10 : height / 2 - 10 + (height / 2 - 10) / (double) max_score * (- _data.get(i)))));
            }
        else
           {
    	    g2.draw (new Line2D.Double (10 + increment * (i + 1), height / 2 - 10,
                    10 + increment * (i + 1),
          (_data.get(i) > max_score ? 10 : (height / 2 - 10) - (height / 2 - 10) / (double) max_score * _data.get(i))));
            }
        }
   }

  public Dimension getPreferredSize ()
  {
   return new Dimension (450, 220);
   }

  public Dimension getMinimumSize ()
  {
   return new Dimension (100, 50);
   }
  }

 class MultipleChoiceValidationDialog extends JDialog implements ActionListener, HyperlinkListener
 {
  /**
   * 
   */
  private static final long serialVersionUID = 1L;

  Vector <Vector <JRadioButton>> _choices;
  String _l1, _l2;
  OntologyInstance _o1, _o2;

  MultipleChoiceValidationDialog (JFrame jf, Vector <PairingScore> v)
  {
   super (jf, "", true);
   _choices = new Vector <Vector <JRadioButton>> ();
   this.setLocationRelativeTo (jf);
   JPanel content = new JPanel (new FlowLayout ());
   for (int i = 0; i < v.size (); i++)
       {
        content.add (makePanel (v.get(i)));
        }

   getContentPane ().add (content, BorderLayout.CENTER);
   JPanel jpOk = new JPanel ();
   JButton jbOk = new JButton ("Ok");
   jbOk.addActionListener (this);
   jpOk.add (jbOk);
   getContentPane ().add(jpOk, BorderLayout.SOUTH);

   GridBagLayout g = new GridBagLayout (); 
   JPanel jpControls = new JPanel (g);
   GridBagConstraints c = new GridBagConstraints ();
   c.gridwidth = GridBagConstraints.REMAINDER;
   c.anchor = GridBagConstraints.WEST;

   JButton jbValAll = new JButton ("Validate all");
   jbValAll.addActionListener (this);
   JButton jbDelayAll = new JButton ("Delay all");
   jbDelayAll.addActionListener (this);
   JButton jbInvAll = new JButton ("Invalidate all");
   jbInvAll.addActionListener (this);

//   g.setConstraints(jbValAll, c);
//   jpControls.add (jbValAll);
//   g.setConstraints(jbDelayAll, c);
//   jpControls.add (jbDelayAll);
//   jpControls.add (jbInvAll);

   JPanel test = new JPanel (new GridLayout (0,1));
   test.add (jbValAll);
   test.add (jbInvAll);
   test.add (jbDelayAll);
   jpControls.add (test);

   getContentPane ().add (jpControls, BorderLayout.WEST);
   pack ();
   }

  /**
   * 
   * @param ps
   * @return
   */
  JPanel makePanel (PairingScore ps)
  {
   JEditorPane editor = new JEditorPane ();
   editor.setEditorKit (new HTMLEditorKit ());
   editor.setEditable (false);

   JPanel main = new JPanel (new BorderLayout ());
//   JPanel jp = new JPanel (new GridLayout (0, 1));
   _o1 = ps.getInstance ();
   String text = "";
//   jp.add (new JLabel (left.getName ()));
   text += _o1.getName () + "<br>";
//   jp.add (new JLabel ("id: " + left.getId ()));
   text += "id: <a href=" + _o1.getId () + ">" + _o1.getId () + "</a><br>";
   _l1 = _o1.getId ();
   _o2 = ps.getOrganCandidate ();
//   jp.add (new JLabel (right.getName ()));
//   jp.add (new JLabel ("id: " + right.getId ()));
   text += _o2.getName () + "<br>";
   text += "id: <a href=" + _o2.getId () + ">" + _o2.getId () + "</a><br>";
   _l2 = _o2.getId ();
   editor.setText (text);
   editor.addHyperlinkListener (this);
//   main.add (jp, BorderLayout.CENTER);
   main.add (editor, BorderLayout.CENTER);

   JPanel jpChoice = new JPanel (new GridLayout (0, 1));
   Vector <JRadioButton> v = new Vector <JRadioButton> ();
   ButtonGroup bg = new ButtonGroup ();
   JRadioButton jrbValidate = new JRadioButton ("Validate");
   jrbValidate.setSelected (true);
   v.add (jrbValidate);
   bg.add (jrbValidate);
   jpChoice.add (jrbValidate);
   JRadioButton jrbDelay = new JRadioButton ("Delay");
   jrbDelay.setSelected (false);
   v.add (jrbDelay);
   bg.add (jrbDelay);
   jpChoice.add (jrbDelay);
   JRadioButton jrbInvalidate = new JRadioButton ("Invalidate");
   jrbInvalidate.setSelected (false);
   v.add (jrbInvalidate);
   bg.add (jrbInvalidate);
   jpChoice.add (jrbInvalidate);
   _choices.add (v);
   main.add (jpChoice, BorderLayout.SOUTH);
   return main;
   }

  public Vector <Integer> getChoices ()
  {
   Vector <Integer> res = new Vector <Integer> ();
   for (int i = 0; i < _choices.size (); i++)
       {
        Vector <JRadioButton> v = _choices.get (i);
        if (v.get (0).isSelected ()) res.add (0);
        else
           if (v.get (1).isSelected ()) res.add (1);
           else
              if (v.get (2).isSelected ()) res.add (2);
        }
   return res;
   }

  /**
   * Events handling.
   */
  public void actionPerformed (ActionEvent e)
  {
   String command = e.getActionCommand ();
   if (command.equals ("Ok")) setVisible (false);
   else
      if (command.equals ("Invalidate all"))
         for (int i = 0; i < _choices.size (); i++)
             {
              Vector <JRadioButton> current = _choices.get (i);
              current.get (2).setSelected (true);
              }
      else
       if (command.equals ("Validate all"))
       {
     for (int i = 0; i < _choices.size (); i++)
     {
      Vector <JRadioButton> current = _choices.get (i);
      current.get (0).setSelected (true);
      }
        }
   if (command.equals ("Delay all"))
   {
 for (int i = 0; i < _choices.size (); i++)
 {
  Vector <JRadioButton> current = _choices.get (i);
  current.get (1).setSelected (true);
  }
    }
   }

  public void hyperlinkUpdate (HyperlinkEvent h)
  {
   //System.err.println ("Event generated: " + h);
   if (h.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
      {
       String link = h.getDescription ();
       System.err.println ("Clicked on " + link);
       if (link.equals (_l1))
       {
        String [] words = _o1.getOntology ().getBaseURI ().split ("/");
          _m.searchInOntologies (link, words [words.length - 1]);
       }
       else
       {
        String [] words = _o2.getOntology ().getBaseURI ().split ("/");
        _m.searchInOntologies (link, words [words.length - 1]);
       }
       }
   }
 }

 /**
  * This function reacts to a click on an hyperlink of the <code>AnnotatedOrganPanel</code>.
  */
 public void hyperlinkUpdate(HyperlinkEvent h)
 {
  //System.err.println ("Event generated: " + h);
  if (h.getEventType() == HyperlinkEvent.EventType.ACTIVATED)
     {
      String link = h.getDescription ();
      System.err.println ("Clicked on " + link);
      String [] words = link.split ("#");
      _m.searchInOntologies (words [1], words [0]);
      }
  }

 class UserDecision
 {
  final static int VALIDATE = 1;
  final static int PARTIAL_VALIDATE = 2;
  final static int INVALIDATE = 3;
  final static int DELAY = 4;
  PairingScore _ps;
  int _type;
  double _decision_score;
  double _base_score;
  int _manual_status; // 0 unknown, 1 validated

  UserDecision (PairingScore ps, double ds, int t, double bs, int ms)
  { _ps = ps; _decision_score = ds; _type = t; _base_score = bs; _manual_status = ms; }

  int getType () { return _type; }

  PairingScore getPairingScore () { return _ps; }
  double getDecisionScore () { return _decision_score; }
  double getBaseScore () { return _base_score; }
  int getManualStatus () { return _manual_status; }
  }

 /**
  * 
  * @param filename
  */
 public void restoreAlignment (final String filename)
 {
  new Thread (new Runnable () { public void run () {

  Vector <PairingScore> list = new Vector <PairingScore> ();
  Vector <Integer> types = new Vector <Integer> ();
  try {
       BufferedReader _handle = new BufferedReader (new FileReader (new File (filename)));
       String str;
       while ((str = _handle.readLine ()) != null)
             {
              String [] words = str.split ("; ");
              Ontology wOnto = _aligner.getWorkingOntology ();
              if (wOnto == null) System.err.println ("working onto is null");
              OntologyInstance left = wOnto.getInstanceById (words [0]);
              OntologyInstance right = _aligner.getReferenceOntology ().getInstanceById (words [1]);
              // PairingScore is created for convenience:
              PairingScore ps = new PairingScore (left, right, new Double (words [2]).doubleValue ());
              list.add (ps);
              types.add (new Integer (words [3]));
              }
       }
  catch (FileNotFoundException e) { /* Nothing to restore */ return; }
  catch (IOException e) { e.printStackTrace (); }
  
  align ();

  // TODO:
  // C'est ici l'erreur. Je devrais faire un getProposition et verifier dans la liste celles qui sont a valider/invalider.

  // Pour l'instant le vecteur v est faux, il contient le PairingScore cr d'apres le fichier. Il devrait contenir celui de l'AnnotatedOrgan retrouv.
  _aoPropositions = _aligner.getPropositions ();
  for (int i = 0; i < list.size (); i++)
      {
       System.err.println (list.get (i).getInstance ().getId () + " and " + list.get (i).getOrganCandidate ().getId () + ": " + list.get (i).getScore () +  " => " + types.get (i));
       // Find in _aoPropositions the corresponding AnnotatedOrgan:
       AnnotatedOrgan ao = null;
       for (int j = 0; j < _aoPropositions.size (); j++)
           {
	    //System.err.println ("Testing " + list.get (i).getInstance ().getId () + " and " + _aoPropositions.get (j).getOrgan ().getId ());
	    if (_aoPropositions.get (j).getOrgan ().getId ().equals (list.get (i).getInstance ().getId ()))
               {
                ao = _aoPropositions.get (j);
                break;
                }
            }
       if (ao != null)
          {
           Vector <PairingScore> v = new Vector <PairingScore> ();
           // TODO: Ce qui vient est faux, il faut rechercher le PairingScore de ao qui contient le bon organCandidate
           v.add (ao.getBestScoreObject ());
           if (types.get (i) == UserDecision.INVALIDATE)
              {
               // Invalidate
               if (ao.getBestScoreObject () != null)
               {
               _decisions.add (new UserDecision (list.get (i), list.get (i).getScore (), UserDecision.INVALIDATE, list.get (i).getIntrinsicScore (), 0));
               _aligner.iterate (new Vector <PairingScore> (), new Vector <PairingScore> (), v, new Vector <PairingScore> ());
               }
               }
           else
              {
               // Validate
               if (types.get (i) == UserDecision.PARTIAL_VALIDATE)
                  {
                   _decisions.add (new UserDecision (list.get (i), list.get (i).getScore (), UserDecision.PARTIAL_VALIDATE, list.get (i).getIntrinsicScore (), 0));
                   _aligner.iterate (new Vector <PairingScore> (), v, new Vector <PairingScore> (), new Vector <PairingScore> ());
                   }
               else
                  if (types.get (i) == UserDecision.VALIDATE)
                     {
                      _decisions.add (new UserDecision (list.get (i), list.get (i).getScore (), UserDecision.VALIDATE, list.get (i).getIntrinsicScore (), 0));
                      _aligner.iterate (v, new Vector <PairingScore> (), new Vector <PairingScore> (), new Vector <PairingScore> ());
                      }
               }
           }
       else
          {
           System.err.println ("No corresponding AnnotatedOrgan found for " + list.get (i).getInstance().getId () + " <=> " + list.get (i).getOrganCandidate ().getId ());
           }
       }
  _jbPrevious.setEnabled (false);
  _aoPropositions = _aligner.getPropositions ();
  _aop = new AnnotatedOrganPanel [_aoPropositions.size ()];
  _psPropositions = new PairingScore [_aoPropositions.size ()];
  if (_aoPropositions.size () > 1) _jbNext.setEnabled (true);
     _current_pos = 0;
  AnnotatedOrgan aoCurrent = (AnnotatedOrgan) _aoPropositions.get (0);
  _aop [_current_pos] = new AnnotatedOrganPanel (AlignmentFrame.this, aoCurrent, findAnnotations (aoCurrent));
  _psPropositions [_current_pos] = aoCurrent.getBestScoreObject ();
  // Il faut vider _jpPropositions :
  _jpPropositions.removeAll ();
  _cl.invalidateLayout (_jpPropositions);
  _jpPropositions.add (aoCurrent.getOrgan ().getId (), _aop [_current_pos]);
  _jpPropositions.revalidate ();
  _aligner.setAnnotations (_annotations);
  setVisible (true);
      }}).start ();

  }

 public void saveDecisions (String _workspace_dirname)
 {
  String filename = _workspace_dirname + File.separatorChar + "decisions";
  try
  {
   FileWriter file = new FileWriter (new File (filename));
   BufferedWriter _bw = new BufferedWriter (file);
   for (int i = 0; i < _decisions.size (); i++)
       {
        _bw.write (_decisions.get (i).getPairingScore ().getInstance ().getId () + "; " +
        	   _decisions.get (i).getPairingScore ().getOrganCandidate ().getId () +
        	   "; " + _decisions.get (i).getDecisionScore () + "; " +
        	   + _decisions.get (i).getType () + "; " + _decisions.get (i).getBaseScore () + "; " + _decisions.get (i).getManualStatus ()
        	   + "; " + _decisions.get (i).getPairingScore ().getInstance ().getName () + "; " + _decisions.get (i).getPairingScore ().getOrganCandidate ().getName ()
        	   + "; " + (_decisions.get (i).getPairingScore ().getInstance ().getName ().equals (_decisions.get (i).getPairingScore ().getOrganCandidate ().getName ()) ? 1 : 0));
        _bw.newLine ();
        }
   _bw.close ();
   }
  catch (Exception e) { e.printStackTrace(); }
  }
}