package visualization;

import java.awt.*;
import javax.swing.BorderFactory;
import javax.swing.border.Border;
import javax.swing.border.EtchedBorder;
import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;

import alignment.AnnotatedOrgan;
import alignment.PairingScore;

import ontology.OntologyInstance;

/**
 * Displays an <code>AnnotatedOrgan</code> in a <code>JPanel</code>.
 */
class AnnotatedOrganDisplayer extends JPanel
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;

 JRadioButton jrValidate, jrInvalidate, jrDelay;

 public final static int VALIDATED = 1;
 public final static int INVALIDATED = 2;
 public final static int DELAYED = 3;

 AnnotatedOrganDisplayer (AnnotatedOrgan ao)
 {
  super ();

  Border b = BorderFactory.createEtchedBorder (EtchedBorder.LOWERED);
  this.setBorder (b);
  // Trois choses a afficher : les deux organes et les cases a cocher pour les choix de
  // l'utilisateur.
  setLayout (new GridLayout (2, 1)); // 2 lignes, 1 colonne.
  // En haut je place mes deux organes, dans un panel :
  JPanel pUp = new JPanel (new FlowLayout ());

  // Affichage de l'organe de l'espece de travail :
  OntologyInstance oi = ao.getOrgan ();
  JPanel jpLeft = new JPanel (new GridLayout (0,1));
  JLabel jlName = new JLabel (oi.getName ());
  jpLeft.add (jlName);
  JLabel jlId = new JLabel (oi.getId ());
  jpLeft.add (jlId);
  JLabel jlScore = new JLabel ("Score: " + ao.getBestScore ());
  jpLeft.add (jlScore);
  pUp.add (jpLeft);

  PairingScore s = ao.getBestScoreObject ();
  JPanel jpRight = new JPanel (new GridLayout (0, 1));
  OntologyInstance oi2 = s.getOrganCandidate ();
  JLabel jlName2 = new JLabel (oi2.getName ());
  jpRight.add (jlName2);
  JLabel jlId2 = new JLabel (oi2.getId ());
  jpRight.add (jlId2);
  pUp.add (jpRight);

  this.add (pUp);

  JPanel jpBottom = new JPanel ();

  JPanel jpButtons = new JPanel (new GridLayout (0, 1));
  jrValidate = new JRadioButton ("Validate");
  jpButtons.add (jrValidate);
  jrDelay = new JRadioButton ("Delay Validation");
  jrDelay.setSelected (true);
  jpButtons.add (jrDelay);
  jrInvalidate = new JRadioButton ("Invalidate");
  jpButtons.add (jrInvalidate);

  ButtonGroup bg = new ButtonGroup ();
  bg.add (jrValidate);
  bg.add (jrDelay);
  bg.add (jrInvalidate);
  jpBottom.add (jpButtons);
  this.add (jpBottom);
  }

 public int getStatus ()
 {
  if (jrValidate.isSelected ()) return VALIDATED;
  else if (jrInvalidate.isSelected ()) return INVALIDATED;
  else return DELAYED;
  }
 }