package visualization;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.*;
import java.awt.GridLayout;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.border.*;
import javax.swing.ButtonGroup;
import javax.swing.BoxLayout;
import javax.swing.JDialog;
import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import javax.swing.JTree;
//import javax.swing.JScrollPane;
import javax.swing.text.html.HTMLEditorKit;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import alignment.AnnotatedOrgan;
import alignment.PairingScore;
import alignment.PairingScoreView;

import ontology.Annotations;
import ontology.OntologyInstance;
import ontology.SpeciesOntology;


/**
 * Displays the data contained in an AnnotatedOrgan. The idea is to give as many
 * informations as possible to help the user to decide about the homology proposed.
 **/
class AnnotatedOrganPanel extends JPanel implements ActionListener
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;

 /**
  * Not used currently. It may be useful to tell to the main window what we have done,
  * especially regarding display of all the possible choices.
  */
 AlignmentFrame _af;
 
 AnnotatedOrgan _ao;
 Annotations _a;

 JRadioButton jrValidate, jrValidateStrict, jrInvalidate, jrDelay, jrDelayIgnore;
 JTextField _jtfAuthor, _jtfLinks;
 JTextArea _jtaComments;
 static String _name = "";

 public final static int VALIDATED = 1;
 public final static int INVALIDATED = 2;
 public final static int DELAYED = 3;
 public final static int VALIDATED_STRICT = 4;
 public final static int DELAY_AND_IGNORE = 5;

 /**
  *
  */
 public AnnotatedOrganPanel (AlignmentFrame af, AnnotatedOrgan ao, Annotations a)
 {
  super (new BorderLayout ());
  _ao = ao;
  _af = af;
  _a = a;
  initUI ();
  }

 private void initUI ()
 {
  JPanel jpMain = new JPanel ();

  jpMain.setLayout (new BoxLayout (jpMain, BoxLayout.PAGE_AXIS));
  
  JPanel jpBoxes = new JPanel (new GridBagLayout ());
  
  // Current organ:
  OntologyInstance oiOrgan = _ao.getOrgan ();
  JPanel organPanel = new JPanel (new GridLayout (0,2));
  Border organBorder = BorderFactory.createLineBorder (Color.black);
  TitledBorder torganBorder = BorderFactory.createTitledBorder (organBorder, "Current organ");
  organPanel.setBorder (torganBorder);
  JEditorPane moPane = new JEditorPane ();
  moPane.setEditorKit (new HTMLEditorKit ());
  moPane.setEditable (false);
  OntologyInstanceView oiv = new OntologyInstanceView (oiOrgan);
  moPane.setText (oiv.displayWithLinks ());
  moPane.addHyperlinkListener (_af);
  organPanel.add (moPane);
  
  // A summary of the hierarchy:
  JTree tree = this.generateTree (oiOrgan);
  organPanel.add (tree);
  GridBagConstraints c = new GridBagConstraints();
  c.fill = GridBagConstraints.BOTH;
  c.gridx = 0;
  c.gridy = 0;
  jpBoxes.add (organPanel, c);

  // Proposed homolog:
  PairingScore ps = _ao.getBestScoreObject ();
  OntologyInstance oiCandidate = ps.getOrganCandidate ();
  JPanel psPanel = new JPanel (new GridLayout (0,2));
  Border psBorder = BorderFactory.createLineBorder (Color.black);
  Border tpsBorder = BorderFactory.createTitledBorder (psBorder, "Putative homolog");
  psPanel.setBorder (tpsBorder);
  JEditorPane ocPane = new JEditorPane ();
  ocPane.addHyperlinkListener (_af);
  ocPane.setEditorKit (new HTMLEditorKit ());
  ocPane.setEditable (false);
  OntologyInstanceView civ = new OntologyInstanceView (oiCandidate);
  ocPane.setText (civ.displayWithLinks ());

  psPanel.add (ocPane);
  
  // A summary of the putative homolog's hierarchy:
  tree = this.generateTree (oiCandidate);
  psPanel.add (tree);

  c.gridx = 1;
  c.gridy = 0;
  jpBoxes.add (psPanel, c);

  // Some stats about current organ's propositions:
  JPanel aoPanel = new JPanel ();
  Border aoBorder = BorderFactory.createLineBorder (Color.black);
  Border taoBorder = BorderFactory.createTitledBorder (aoBorder, "Current Org. stats.");
  aoPanel.setBorder (taoBorder);
  JEditorPane aoPane = new JEditorPane ();
  aoPane.setEditorKit (new HTMLEditorKit ());
  aoPane.setEditable (false);
  AnnotatedOrganView aov = new AnnotatedOrganView (_ao);
  aoPane.setText (aov.display ());
  aoPanel.add (aoPane);
  c.gridx = 0;
  c.gridy = 1;
  jpBoxes.add (aoPanel, c);

  // Score & comments:
  JPanel jpBottomRight = new JPanel (new GridBagLayout ());
  GridBagConstraints c2 = new GridBagConstraints();
  c2.fill = GridBagConstraints.HORIZONTAL;
  c2.gridx = 0;
  c2.gridy = 0;

  // Score:
  JPanel sPanel = new JPanel ();
  Border sBorder = BorderFactory.createLineBorder (Color.black);
  Border tsBorder = BorderFactory.createTitledBorder (sBorder, "Score");
  sPanel.setBorder (tsBorder);
  PairingScoreView psv = new PairingScoreView (ps);
  JEditorPane sPane = new JEditorPane ();
  sPane.setEditorKit (new HTMLEditorKit ());
  sPane.setEditable (false);
  sPane.setText (psv.display ());
  sPanel.add (sPane);
  c.gridx = 1;
  c.gridy = 1;
  jpBottomRight.add (sPanel, c2);

  // Comments:
  JPanel jpAnnot = new JPanel (new GridBagLayout ());
  GridBagConstraints c3 = new GridBagConstraints();
  c3.fill = GridBagConstraints.HORIZONTAL;
  c3.gridx = 0;
  c3.gridy = 0;
  Border aBorder = BorderFactory.createLineBorder (Color.black);
  Border taBorder = BorderFactory.createTitledBorder (aBorder, "Annotations");
  jpAnnot.setBorder (taBorder);

  JPanel jpName = new JPanel (new FlowLayout ());
  JLabel jla = new JLabel ("Author");
  jpName.add (jla);
  _jtfAuthor = new JTextField ("", 20);
  if (_a != null)
  {
     _jtfAuthor.setText (_a.getAuthor ());
     // test:
     _jtfAuthor.setText (_name);
  }
  else
     _jtfAuthor.setText (_name);
  jpName.add (_jtfAuthor);
  jpAnnot.add (jpName, c3);

  JPanel jpLinks = new JPanel (new FlowLayout ());
  JLabel jll = new JLabel ("Links");
  jpLinks.add (jll);
  _jtfLinks = new JTextField ("", 20);
  if (_a != null)
	     _jtfLinks.setText (_a.getLink ());
  jpLinks.add (_jtfLinks);
  c3.gridx = 0;
  c3.gridy = 1;
  jpAnnot.add (jpLinks, c3);

  JPanel jpComments = new JPanel (new FlowLayout ());
  JLabel jlp = new JLabel ("Comments");
  jpComments.add (jlp);
  _jtaComments = new JTextArea (5, 30);
  jpComments.add (_jtaComments);
  c3.gridx = 0;
  c3.gridy = 2;
  c3.fill = GridBagConstraints.BOTH;
  jpAnnot.add (jpComments, c3);

  c2.fill = GridBagConstraints.BOTH;
  c2.gridx = 0;
  c2.gridy = 1;
  jpBottomRight.add (jpAnnot, c2);
  jpBoxes.add (jpBottomRight, c);

  jpMain.add (jpBoxes);
  
  // En bas, les radio boutons de la validation :
  Dimension dim = new Dimension (200, 100);
  JPanel jrPanel = new JPanel (new GridLayout (0, 1));
  jrPanel.setMaximumSize (dim);
  jrPanel.setMinimumSize (dim);
  jrPanel.setPreferredSize (dim);
  jrValidateStrict = new JRadioButton ("Validate Homology");
  jrPanel.add (jrValidateStrict);
  jrValidate = new JRadioButton ("Validate Partial Homology");
  jrPanel.add (jrValidate);
  jrDelay = new JRadioButton ("Delay");
  jrPanel.add (jrDelay);
  jrDelay.setSelected (true);
  jrDelayIgnore = new JRadioButton ("Delay and ignore");
  jrPanel.add (jrDelayIgnore);
  jrInvalidate = new JRadioButton ("Invalidate");
  jrPanel.add (jrInvalidate);
  ButtonGroup bg = new ButtonGroup ();
  bg.add (jrValidateStrict);
  bg.add (jrValidate);
  bg.add (jrDelay);
  bg.add (jrDelayIgnore);
  bg.add (jrInvalidate);
  jpMain.add (jrPanel);

  this.add (jpMain, BorderLayout.CENTER);

  Border b = BorderFactory.createEtchedBorder (EtchedBorder.LOWERED);
  this.setBorder (b);
  }

 /**
  *
  */
 private String formatName (String name)
 {
  String [] words = name.split (" ");
  if (name.length() > 20)
     {
      int ind = (words.length + 1) / 2;
      String new_name = "<html>";
      for (int i = 0; i < ind - 1; i ++) new_name += words [i] + " ";
      new_name += words [ind - 1];
      new_name += "<br>";
      for (int i = ind; i < words.length; i++) new_name += words [i] + " ";
      new_name += "</html>";
      name = new_name;
      }
  return name;
  }
 
private JTree generateTree (OntologyInstance oiOrgan)
{
 JTree jt = null;

 SpeciesOntology sp1 = (SpeciesOntology) oiOrgan.getOntology ();
 Vector <String> o = oiOrgan.getObjectOfPropertyByPropertyIdAsVector (sp1.getPartOfPropertyId ());
 Vector<DefaultMutableTreeNode> leaves = new Vector<DefaultMutableTreeNode> ();
 if (o == null)
    {
     DefaultMutableTreeNode root = new DefaultMutableTreeNode ();
     DefaultMutableTreeNode leaf = new DefaultMutableTreeNode (formatName (oiOrgan.getName ()));
     root.add (leaf);
     DefaultTreeModel tm = new DefaultTreeModel (root);
     JTree tree = new JTree (tm);
     tree.setSelectionPath (new TreePath (leaf.getPath ()));
     tree.setRootVisible (false);
     tree.setRowHeight (0);
     return tree;
     }
 else
    {
     DefaultMutableTreeNode root = new DefaultMutableTreeNode ();
     for (int i = 0; i < o.size (); i++)
         {
          // Ajouter les parents de o comme fils de root :
          OntologyInstance oiParent = sp1.getInstanceById((String) o.get(i));
          DefaultMutableTreeNode dmtnParent = new DefaultMutableTreeNode (formatName (oiParent.getName ()));
          DefaultMutableTreeNode leaf2 = new DefaultMutableTreeNode (formatName (oiOrgan.getName ()));
          dmtnParent.add (leaf2);
          leaves.add (leaf2);
	       Vector <String> p = oiParent.getObjectOfPropertyByPropertyIdAsVector (sp1.getPartOfPropertyId ());
          if (p == null)
             {
              root.add (dmtnParent);
              }
          else
             {
              for (int j = 0; j < p.size (); j++)
                  {
                   DefaultMutableTreeNode grand_parent = new DefaultMutableTreeNode (formatName (sp1.getInstanceById((String) p.get (j)).getName ()));
                   DefaultMutableTreeNode leaf3 = new DefaultMutableTreeNode (formatName (oiOrgan.getName ()));
                   DefaultMutableTreeNode dmtnParent2 = new DefaultMutableTreeNode (dmtnParent.getUserObject ());
                   dmtnParent2.add (leaf3);
                   leaves.add (leaf3);
                   grand_parent.add (dmtnParent2);
                   root.add (grand_parent);
                   }
              }
          }
     DefaultTreeModel tm = new DefaultTreeModel (root);
     jt = new JTree (tm);
     jt.setRootVisible (false);
     for (int i = 0; i < leaves.size (); i++)
         jt.setSelectionPath (new TreePath (leaves.get (i).getPath ()));
     }
 jt.setRowHeight (0);
 return jt;
 }

 /**
  *
  **/
 public void actionPerformed (ActionEvent e)
 {
  }

 /**
  *
  **/
 // Cette fonction ne peut etre que temporaire. Deja, je voudrais directement
 // transmettre des PairingScore, afin de traiter directement l'objet qui a ete valide
 // par l'utilisateur.
 // Ensuite, il est possible que l'on renvoie plusieurs PairingScore, si l'utilisateur a
 // voulu acceder a la liste complete des propositions pour l'organe en cours. Du coup,
 // je verrais plutot 3 fonctions du genre 'getValidatedScores', ...
 public int getStatus ()
 {
  if (jrValidate.isSelected ()) return VALIDATED;
  else if (jrValidateStrict.isSelected ()) return VALIDATED_STRICT;
  else if (jrInvalidate.isSelected ()) return INVALIDATED;
  else if (jrDelayIgnore.isSelected ()) return DELAY_AND_IGNORE;
  else return DELAYED;
  }

 public void validateHomology ()
 {
  jrValidateStrict.setSelected (true);
  }

 public void invalidateHomology ()
 {
  jrInvalidate.setSelected (true);
  }

 /**
  * 
  * @return an <code>Annotations</code> object for the current pairing.
  */
 public Annotations getAnnotations ()
 {
  if (_a != null)
     {
      _a.setLink (_jtfLinks.getText ());
      if (! _jtfAuthor.getText ().equals (""))
         setAuthorName (_jtfAuthor.getText ());
      _a.setAuthor(_jtfAuthor.getText ());
      _a.setComments(_jtaComments.getText ());
      _a.setStatus(getStatus ());
      return _a;
      }
  else
     {
      //if ((! _jtfAuthor.getText ().equals("")) || (! _jtfLinks.getText ().equals("")) || (! _jtaComments.getText ().equals("")))
      if ((!(_jtfAuthor.getText () == null)) || (!(_jtfLinks.getText () == null)) || (!(_jtaComments.getText () == null)))
         {
          int status = getStatus ();
          String a = _jtfAuthor.getText ();
          String l = _jtfLinks.getText ();
          String c = _jtaComments.getText ();
          return new Annotations (_ao.getBestScoreObject ().getOrganCandidate (), _ao.getOrgan (), a, l, c, status);
          }
      else
	     return null;
   }
  }
 public static void setAuthorName (String name) { _name = name; }
 }


class AllChoicesDialog extends JDialog
{
 JFrame _parent;
 /**
  * 
  */
 private static final long serialVersionUID = 1L;

 AllChoicesDialog (JFrame parent, AnnotatedOrgan ao)
 {
  super (parent, true);
  parent = _parent;
  }
 }