package visualization;

import java.awt.Component;

import javax.swing.ImageIcon;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeCellRenderer;



/**
 * This class modifies the default tree renderer in order to add icons. We currently add
 * icons for classes, properties and instances.
 * @author Gilles Parmentier
 */
class OntologyCellRenderer extends DefaultTreeCellRenderer
{
 /**
  * 
  */
 private static final long serialVersionUID = 1L;
 ImageIcon _class_icon, _instance_icon, _property_icon;
 
 public OntologyCellRenderer ()
 {
  ClassLoader cl = this.getClass ().getClassLoader ();
  _class_icon = new ImageIcon (cl.getResource ("visualization/Class.gif"));
  _instance_icon = new ImageIcon (cl.getResource ("visualization/Instance.gif"));
  _property_icon = new ImageIcon (cl.getResource ("visualization/Property.gif"));
  }

 /**
  * Where the job is actually done. I use the function <code>getName()</code> of the
  * <code>Class</code> class to determine what is the class of the current node.
  */

 public Component getTreeCellRendererComponent (JTree tree, Object value, boolean sel,
                                boolean expanded, boolean leaf, int row, boolean hasFocus)
 {
  super.getTreeCellRendererComponent (tree, value, sel, expanded, leaf, row, hasFocus);
  Object o = ((DefaultMutableTreeNode) value).getUserObject ();
  if (o == null) return this;
  //System.out.println ("OntologyCellRenderer, type is: " + o.getClass ().getName ());
  if (o.getClass ().getName ().equals ("visualization.OntologyInstanceView"))
     setIcon (_instance_icon);
  else
     if (o.getClass ().getName ().equals ("visualization.OntologyPropertyView"))
        setIcon (_property_icon);
     else
        setIcon (_class_icon);
  return this;
  }

 };